@@ -53,7 +53,7 @@ import org.apache.activemq.util.URISupport.CompositeData;
  * TopicConnectionFactory. You can use this connection to create both
  * QueueConnections and TopicConnections.
  * 
- * @version $Revision: 1.9 $
+ * 
  * @see javax.jms.ConnectionFactory
  */
 public class ActiveMQConnectionFactory extends JNDIBaseStorable implements ConnectionFactory, QueueConnectionFactory, TopicConnectionFactory, StatsCapable, Cloneable {
@@ -83,6 +83,8 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
 
     private IdGenerator clientIdGenerator;
     private String clientIDPrefix;
+    private IdGenerator connectionIdGenerator;
+    private String connectionIDPrefix;
 
     // client policies
     private ActiveMQPrefetchPolicy prefetchPolicy = new ActiveMQPrefetchPolicy();
@@ -92,6 +94,7 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
 
     private boolean disableTimeStampsByDefault;
     private boolean optimizedMessageDispatch = true;
+    private long optimizeAcknowledgeTimeOut = 300;
     private boolean copyMessageOnSend = true;
     private boolean useCompression;
     private boolean objectMessageSerializationDefered;
@@ -116,6 +119,8 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
     private boolean checkForDuplicates = true;
     private ClientInternalExceptionListener clientInternalExceptionListener;
     private boolean messagePrioritySupported = true;
+    private boolean transactedIndividualAck = false;
+    private boolean nonBlockingRedelivery = false;
 
     // /////////////////////////////////////////////
     //
@@ -288,7 +293,8 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
     }
 
     protected ActiveMQConnection createActiveMQConnection(Transport transport, JMSStatsImpl stats) throws Exception {
-        ActiveMQConnection connection = new ActiveMQConnection(transport, getClientIdGenerator(), stats);
+        ActiveMQConnection connection = new ActiveMQConnection(transport, getClientIdGenerator(),
+                getConnectionIdGenerator(), stats);
         return connection;
     }
 
@@ -321,6 +327,8 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
         connection.setConsumerFailoverRedeliveryWaitPeriod(getConsumerFailoverRedeliveryWaitPeriod());
         connection.setCheckForDuplicates(isCheckForDuplicates());
         connection.setMessagePrioritySupported(isMessagePrioritySupported());
+        connection.setTransactedIndividualAck(isTransactedIndividualAck());
+        connection.setNonBlockingRedelivery(isNonBlockingRedelivery());
         if (transportListener != null) {
             connection.addTransportListener(transportListener);
         }
@@ -703,6 +711,8 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
         props.setProperty("auditMaximumProducerNumber", Integer.toString(getAuditMaximumProducerNumber()));
         props.setProperty("checkForDuplicates", Boolean.toString(isCheckForDuplicates()));
         props.setProperty("messagePrioritySupported", Boolean.toString(isMessagePrioritySupported()));
+        props.setProperty("transactedIndividualAck", Boolean.toString(isTransactedIndividualAck()));
+        props.setProperty("nonBlockingRedelivery", Boolean.toString(isNonBlockingRedelivery()));
     }
 
     public boolean isUseCompression() {
@@ -800,6 +810,18 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
         this.optimizeAcknowledge = optimizeAcknowledge;
     }
 
+    /**
+     * The max time in milliseconds between optimized ack batches
+     * @param optimizeAcknowledgeTimeOut
+     */
+    public void setOptimizeAcknowledgeTimeOut(int optimizeAcknowledgeTimeOut) {
+        this.optimizeAcknowledgeTimeOut =  optimizeAcknowledgeTimeOut;
+    }
+
+    public long getOptimizeAcknowledgeTimeOut() {
+        return optimizeAcknowledgeTimeOut;
+    }
+
     public boolean isNestedMapAndListEnabled() {
         return nestedMapAndListEnabled;
     }
@@ -843,6 +865,29 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
         this.clientIdGenerator = clientIdGenerator;
     }
 
+    /**
+     * Sets the prefix used by connection id generator
+     * @param connectionIDPrefix
+     */
+    public void setConnectionIDPrefix(String connectionIDPrefix) {
+        this.connectionIDPrefix = connectionIDPrefix;
+    }
+
+    protected synchronized IdGenerator getConnectionIdGenerator() {
+        if (connectionIdGenerator == null) {
+            if (connectionIDPrefix != null) {
+                connectionIdGenerator = new IdGenerator(connectionIDPrefix);
+            } else {
+                connectionIdGenerator = new IdGenerator();
+            }
+        }
+        return connectionIdGenerator;
+    }
+
+    protected void setConnectionIdGenerator(IdGenerator connectionIdGenerator) {
+        this.connectionIdGenerator = connectionIdGenerator;
+    }
+
     /**
      * @return the statsEnabled
      */
@@ -980,4 +1025,32 @@ public class ActiveMQConnectionFactory extends JNDIBaseStorable implements Conne
     public void setCheckForDuplicates(boolean checkForDuplicates) {
         this.checkForDuplicates = checkForDuplicates;
     }
+
+    public boolean isTransactedIndividualAck() {
+         return transactedIndividualAck;
+     }
+
+     /**
+      * when true, submit individual transacted acks immediately rather than with transaction completion.
+      * This allows the acks to represent delivery status which can be persisted on rollback
+      * Used in conjunction with org.apache.activemq.store.kahadb.KahaDBPersistenceAdapter#setRewriteOnRedelivery(boolean)  true
+      */
+     public void setTransactedIndividualAck(boolean transactedIndividualAck) {
+         this.transactedIndividualAck = transactedIndividualAck;
+     }
+
+
+     public boolean isNonBlockingRedelivery() {
+         return nonBlockingRedelivery;
+     }
+
+     /**
+      * When true a MessageConsumer will not stop Message delivery before re-delivering Messages
+      * from a rolled back transaction.  This implies that message order will not be preserved and
+      * also will result in the TransactedIndividualAck option to be enabled.
+      */
+     public void setNonBlockingRedelivery(boolean nonBlockingRedelivery) {
+         this.nonBlockingRedelivery = nonBlockingRedelivery;
+     }
+
 }
