@@ -23,6 +23,7 @@ import java.net.URISyntaxException;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -73,7 +74,9 @@ import org.apache.activemq.broker.region.virtual.VirtualDestinationInterceptor;
 import org.apache.activemq.broker.region.virtual.VirtualTopic;
 import org.apache.activemq.broker.scheduler.SchedulerBroker;
 import org.apache.activemq.command.ActiveMQDestination;
+import org.apache.activemq.command.ActiveMQQueue;
 import org.apache.activemq.command.BrokerId;
+import org.apache.activemq.filter.DestinationFilter;
 import org.apache.activemq.network.ConnectionFilter;
 import org.apache.activemq.network.DiscoveryNetworkConnector;
 import org.apache.activemq.network.NetworkConnector;
@@ -93,14 +96,16 @@ import org.apache.activemq.transport.TransportServer;
 import org.apache.activemq.transport.vm.VMTransportFactory;
 import org.apache.activemq.usage.SystemUsage;
 import org.apache.activemq.util.*;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.slf4j.MDC;
+
 /**
  * Manages the lifecycle of an ActiveMQ Broker. A BrokerService consists of a
  * number of transport connectors, network connectors and a bunch of properties
  * which can be used to configure the broker as its lazily created.
- * 
- * @version $Revision: 1.1 $
+ *
+ *
  * @org.apache.xbean.XBean
  */
 public class BrokerService implements Service {
@@ -108,7 +113,7 @@ public class BrokerService implements Service {
     public static final String DEFAULT_PORT = "61616";
     public static final String LOCAL_HOST_NAME;
     public static final String DEFAULT_BROKER_NAME = "localhost";
-    private static final Log LOG = LogFactory.getLog(BrokerService.class);
+    private static final Logger LOG = LoggerFactory.getLogger(BrokerService.class);
     private static final long serialVersionUID = 7353129142305630237L;
     private boolean useJmx = true;
     private boolean enableStatistics = true;
@@ -194,10 +199,16 @@ public class BrokerService implements Service {
     private ThreadPoolExecutor executor;
     private boolean slave = true;
     private int schedulePeriodForDestinationPurge=5000;
+    private int maxPurgedDestinationsPerSweep = 0;
     private BrokerContext brokerContext;
     private boolean networkConnectorStartAsync = false;
+    private boolean allowTempAutoCreationOnSend;
 
-	static {
+    private int offlineDurableSubscriberTimeout = -1;
+    private int offlineDurableSubscriberTaskSchedule = 300000;
+    private DestinationFilter virtualConsumerDestinationFilter;
+
+    static {
         String localHostName = "localhost";
         try {
             localHostName =  InetAddressUtil.getLocalHostName();
@@ -214,7 +225,7 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new transport connector for the given bind address
-     * 
+     *
      * @return the newly created and added transport connector
      * @throws Exception
      */
@@ -224,7 +235,7 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new transport connector for the given bind address
-     * 
+     *
      * @return the newly created and added transport connector
      * @throws Exception
      */
@@ -234,7 +245,7 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new transport connector for the given TransportServer transport
-     * 
+     *
      * @return the newly created and added transport connector
      * @throws Exception
      */
@@ -244,7 +255,7 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new transport connector
-     * 
+     *
      * @return the transport connector
      * @throws Exception
      */
@@ -255,7 +266,7 @@ public class BrokerService implements Service {
 
     /**
      * Stops and removes a transport connector from the broker.
-     * 
+     *
      * @param connector
      * @return true if the connector has been previously added to the broker
      * @throws Exception
@@ -270,7 +281,7 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new network connector using the given discovery address
-     * 
+     *
      * @return the newly created and added network connector
      * @throws Exception
      */
@@ -280,7 +291,7 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new proxy connector using the given bind address
-     * 
+     *
      * @return the newly created and added network connector
      * @throws Exception
      */
@@ -290,22 +301,18 @@ public class BrokerService implements Service {
 
     /**
      * Adds a new network connector using the given discovery address
-     * 
+     *
      * @return the newly created and added network connector
      * @throws Exception
      */
     public NetworkConnector addNetworkConnector(URI discoveryAddress) throws Exception {
-        if (!isAdvisorySupport()) {
-            throw new javax.jms.IllegalStateException(
-                    "Networks require advisory messages to function - advisories are currently disabled");
-        }
         NetworkConnector connector = new DiscoveryNetworkConnector(discoveryAddress);
         return addNetworkConnector(connector);
     }
 
     /**
      * Adds a new proxy connector using the given bind address
-     * 
+     *
      * @return the newly created and added network connector
      * @throws Exception
      */
@@ -415,7 +422,7 @@ public class BrokerService implements Service {
 
     public void masterFailed() {
         if (shutdownOnMasterFailure) {
-            LOG.fatal("The Master has failed ... shutting down");
+            LOG.error("The Master has failed ... shutting down");
             try {
                 stop();
             } catch (Exception e) {
@@ -436,6 +443,20 @@ public class BrokerService implements Service {
         return started.get();
     }
 
+    /**
+     * Forces a start of the broker. 
+     * By default a BrokerService instance that was 
+     * previously stopped using BrokerService.stop() cannot be restarted
+     * using BrokerService.start(). 
+     * This method enforces a restart. 
+     * It is not recommended to force a restart of the broker and will not work
+     * for most but some very trivial broker configurations. 
+     * For restarting a broker instance we recommend to first call stop() on
+     * the old instance and then recreate a new BrokerService instance.
+     * 
+     * @param force - if true enforces a restart.
+     * @throws Exception
+     */
     public void start(boolean force) throws Exception {
         forceStart = force;
         stopped.set(false);
@@ -471,13 +492,16 @@ public class BrokerService implements Service {
             // throw new IllegalStateException("Allready started.");
             return;
         }
+
+        MDC.put("activemq.broker", brokerName);
+
         try {
-        	if (systemExitOnShutdown && useShutdownHook) {
-        		throw new ConfigurationException("'useShutdownHook' property cannot be be used with 'systemExitOnShutdown', please turn it off (useShutdownHook=false)");
-        	}
+            if (systemExitOnShutdown && useShutdownHook) {
+                throw new ConfigurationException("'useShutdownHook' property cannot be be used with 'systemExitOnShutdown', please turn it off (useShutdownHook=false)");
+            }
             processHelperProperties();
             if (isUseJmx()) {
-            	startManagementContext();
+                startManagementContext();
             }
             getPersistenceAdapter().setUsageManager(getProducerSystemUsage());
             getPersistenceAdapter().setBrokerName(getBrokerName());
@@ -491,15 +515,15 @@ public class BrokerService implements Service {
             addShutdownHook();
             getBroker().start();
             if (isUseJmx()) {
-            	if (getManagementContext().isCreateConnector() && !getManagementContext().isConnectorStarted()) {
-            		// try to restart management context
-            		// typical for slaves that use the same ports as master
-            		managementContext.stop();
-            		startManagementContext();
-            	}
+                if (getManagementContext().isCreateConnector() && !getManagementContext().isConnectorStarted()) {
+                    // try to restart management context
+                    // typical for slaves that use the same ports as master
+                    managementContext.stop();
+                    startManagementContext();
+                }
                 ManagedRegionBroker managedBroker = (ManagedRegionBroker) regionBroker;
                 managedBroker.setContextBroker(broker);
-            	adminView.setBroker(managedBroker);
+                adminView.setBroker(managedBroker);
             }
             BrokerRegistry.getInstance().bind(getBrokerName(), this);
             // see if there is a MasterBroker service and if so, configure
@@ -522,13 +546,13 @@ public class BrokerService implements Service {
                 brokerId = broker.getBrokerId();
             }
             if (ioExceptionHandler == null) {
-            	setIoExceptionHandler(new DefaultIOExceptionHandler());
+                setIoExceptionHandler(new DefaultIOExceptionHandler());
             }
             LOG.info("ActiveMQ JMS Message Broker (" + getBrokerName() + ", " + brokerId + ") started");
             getBroker().brokerServiceStarted();
             startedLatch.countDown();
         } catch (Exception e) {
-            LOG.error("Failed to start ActiveMQ JMS Message Broker. Reason: " + e, e);
+            LOG.error("Failed to start ActiveMQ JMS Message Broker (" + getBrokerName() + ", " + brokerId + "). Reason: " + e, e);
             try {
                 if (!stopped.get()) {
                     stop();
@@ -537,6 +561,8 @@ public class BrokerService implements Service {
                 LOG.warn("Failed to stop broker after failure in start ", ex);
             }
             throw e;
+        } finally {
+            MDC.remove("activemq.broker");
         }
     }
 
@@ -550,18 +576,24 @@ public class BrokerService implements Service {
         if (!started.get()) {
             return;
         }
-        
+
+        MDC.put("activemq.broker", brokerName);
+
         if (systemExitOnShutdown) {
-        	new Thread() {
-        		@Override
+            new Thread() {
+                @Override
                 public void run() {
-        			System.exit(systemExitOnShutdownExitCode);
-        		}
-        	}.start();
+                    System.exit(systemExitOnShutdownExitCode);
+                }
+            }.start();
         }
-        
+
         LOG.info("ActiveMQ Message Broker (" + getBrokerName() + ", " + brokerId + ") is shutting down");
         removeShutdownHook();
+        if (this.scheduler != null) {
+            this.scheduler.stop();
+            this.scheduler = null;
+        }
         ServiceStopper stopper = new ServiceStopper();
         if (services != null) {
             for (Service service : services) {
@@ -578,22 +610,25 @@ public class BrokerService implements Service {
             stopper.stop(broker);
             broker = null;
         }
-        
+
         if (tempDataStore != null) {
             tempDataStore.stop();
             tempDataStore = null;
         }
-        stopper.stop(persistenceAdapter);
-        persistenceAdapter = null;
-        slave = true;
-        if (isUseJmx()) {
-            stopper.stop(getManagementContext());
-            managementContext = null;
+        try {
+            stopper.stop(persistenceAdapter);
+            persistenceAdapter = null;
+            slave = true;
+            if (isUseJmx()) {
+                stopper.stop(getManagementContext());
+                managementContext = null;
+            }
+            // Clear SelectorParser cache to free memory
+            SelectorParser.clearCache();
+        } finally {
+            stopped.set(true);
+            stoppedLatch.countDown();
         }
-        // Clear SelectorParser cache to free memory
-        SelectorParser.clearCache();
-        stopped.set(true);
-        stoppedLatch.countDown();
         if (masterConnectorURI == null) {
             // master start has not finished yet
             if (slaveStartSignal.getCount() == 1) {
@@ -616,18 +651,14 @@ public class BrokerService implements Service {
             this.taskRunnerFactory.shutdown();
             this.taskRunnerFactory = null;
         }
-        if (this.scheduler != null) {
-            this.scheduler.stop();
-            this.scheduler = null;
-        }
         if (this.executor != null) {
             this.executor.shutdownNow();
             this.executor = null;
         }
-        
+
         this.destinationInterceptors = null;
         this.destinationFactory = null;
-        
+
         LOG.info("ActiveMQ JMS Message Broker (" + getBrokerName() + ", " + brokerId + ") stopped");
         synchronized (shutdownHooks) {
             for (Runnable hook : shutdownHooks) {
@@ -638,10 +669,12 @@ public class BrokerService implements Service {
                 }
             }
         }
-        
+
+        MDC.remove("activemq.broker");
+
         stopper.throwFirstException();
     }
-    
+
     public boolean checkQueueSize(String queueName) {
         long count = 0;
         long queueSize = 0;
@@ -669,7 +702,7 @@ public class BrokerService implements Service {
      * client should failover to other broker and pending messages should be
      * forwarded. if no pending messages, the method finally call stop to stop
      * the broker.
-     * 
+     *
      * @param connectorName
      * @param queueName
      * @param timeout
@@ -780,7 +813,7 @@ public class BrokerService implements Service {
 
     /**
      * Sets the name of this broker; which must be unique in the network
-     * 
+     *
      * @param brokerName
      */
     public void setBrokerName(String brokerName) {
@@ -813,7 +846,7 @@ public class BrokerService implements Service {
     /**
      * Sets the directory in which the data files will be stored by default for
      * the JDBC and Journal persistence adaptors.
-     * 
+     *
      * @param dataDirectory
      *            the directory to store data files
      */
@@ -824,7 +857,7 @@ public class BrokerService implements Service {
     /**
      * Sets the directory in which the data files will be stored by default for
      * the JDBC and Journal persistence adaptors.
-     * 
+     *
      * @param dataDirectoryFile
      *            the directory to store data files
      */
@@ -896,7 +929,7 @@ public class BrokerService implements Service {
             }
             return systemUsage;
         } catch (IOException e) {
-            LOG.fatal("Cannot create SystemUsage", e);
+            LOG.error("Cannot create SystemUsage", e);
             throw new RuntimeException("Fatally failed to create SystemUsage" + e.getMessage());
         }
     }
@@ -983,7 +1016,7 @@ public class BrokerService implements Service {
 
     /**
      * Sets the persistence adaptor implementation to use for this broker
-     * 
+     *
      * @throws IOException
      */
     public void setPersistenceAdapter(PersistenceAdapter persistenceAdapter) throws IOException {
@@ -1089,15 +1122,17 @@ public class BrokerService implements Service {
         }
         return null;
     }
-    
+
     public Map<String, String> getTransportConnectorURIsAsMap() {
         Map<String, String> answer = new HashMap<String, String>();
         for (TransportConnector connector : transportConnectors) {
             try {
                 URI uri = connector.getConnectUri();
-                String scheme = uri.getScheme();
-                if (scheme != null) {
-                    answer.put(scheme.toLowerCase(), uri.toString());
+                if (uri != null) {
+                    String scheme = uri.getScheme();
+                    if (scheme != null) {
+                        answer.put(scheme.toLowerCase(), uri.toString());
+                    }
                 }
             } catch (Exception e) {
                 LOG.debug("Failed to read URI to build transportURIsAsMap", e);
@@ -1202,8 +1237,8 @@ public class BrokerService implements Service {
     /**
      * Sets the transport connectors which this broker will listen on for new
      * clients
-     * 
-     * @org.apache.xbean.Property 
+     *
+     * @org.apache.xbean.Property
      *                            nestedType="org.apache.activemq.broker.TransportConnector"
      */
     public void setTransportConnectors(List<TransportConnector> transportConnectors) throws Exception {
@@ -1224,8 +1259,8 @@ public class BrokerService implements Service {
     /**
      * Sets the network connectors which this broker will use to connect to
      * other brokers in a federated network
-     * 
-     * @org.apache.xbean.Property 
+     *
+     * @org.apache.xbean.Property
      *                            nestedType="org.apache.activemq.network.NetworkConnector"
      */
     public void setNetworkConnectors(List networkConnectors) throws Exception {
@@ -1284,7 +1319,7 @@ public class BrokerService implements Service {
 
     /**
      * Delete all messages from the persistent store
-     * 
+     *
      * @throws IOException
      */
     public void deleteAllMessages() throws IOException {
@@ -1317,11 +1352,11 @@ public class BrokerService implements Service {
     public void setVmConnectorURI(URI vmConnectorURI) {
         this.vmConnectorURI = vmConnectorURI;
     }
-    
+
     public String getDefaultSocketURIString() {
-       
+
             if (started.get()) {
-                if (this.defaultSocketURIString ==null) {
+                if (this.defaultSocketURIString == null) {
                     for (TransportConnector tc:this.transportConnectors) {
                         String result = null;
                         try {
@@ -1330,10 +1365,19 @@ public class BrokerService implements Service {
                           LOG.warn("Failed to get the ConnectURI for "+tc,e);
                         }
                         if (result != null) {
-                            this.defaultSocketURIString =result;
-                            break;
+                            // find first publishable uri
+                            if (tc.isUpdateClusterClients() || tc.isRebalanceClusterClients()) {
+                                this.defaultSocketURIString = result;
+                                break;
+                            } else {
+                            // or use the first defined
+                                if (this.defaultSocketURIString == null) {
+                                    this.defaultSocketURIString = result;
+                                }
+                            }
                         }
                     }
+
                 }
                 return this.defaultSocketURIString;
             }
@@ -1443,6 +1487,7 @@ public class BrokerService implements Service {
                 }
                 this.tempDataStore = new PListStore();
                 this.tempDataStore.setDirectory(getTmpDataDirectory());
+                configureService(tempDataStore);
                 this.tempDataStore.start();
             } catch (Exception e) {
                 throw new RuntimeException(e);
@@ -1457,6 +1502,14 @@ public class BrokerService implements Service {
      */
     public void setTempDataStore(PListStore tempDataStore) {
         this.tempDataStore = tempDataStore;
+        configureService(tempDataStore);
+        try {
+            tempDataStore.start();
+        } catch (Exception e) {
+            RuntimeException exception = new RuntimeException("Failed to start provided temp data store: " + tempDataStore, e);
+            LOG.error(exception.getLocalizedMessage(), e);
+            throw exception;
+        }
     }
 
     public int getPersistenceThreadPriority() {
@@ -1590,7 +1643,7 @@ public class BrokerService implements Service {
     /**
      * Handles any lazy-creation helper properties which are added to make
      * things easier to configure inside environments such as Spring
-     * 
+     *
      * @throws Exception
      */
     protected void processHelperProperties() throws Exception {
@@ -1628,7 +1681,7 @@ public class BrokerService implements Service {
         }
     }
 
-    protected void stopAllConnectors(ServiceStopper stopper) {
+    public void stopAllConnectors(ServiceStopper stopper) {
         for (Iterator<NetworkConnector> iter = getNetworkConnectors().iterator(); iter.hasNext();) {
             NetworkConnector connector = iter.next();
             unregisterNetworkConnectorMBean(connector);
@@ -1761,7 +1814,7 @@ public class BrokerService implements Service {
 
     /**
      * Factory method to create a new broker
-     * 
+     *
      * @throws Exception
      * @throws
      * @throws
@@ -1798,7 +1851,7 @@ public class BrokerService implements Service {
     /**
      * Factory method to create the core region broker onto which interceptors
      * are added
-     * 
+     *
      * @throws Exception
      */
     protected Broker createRegionBroker() throws Exception {
@@ -1826,6 +1879,7 @@ public class BrokerService implements Service {
         regionBroker.setKeepDurableSubsActive(keepDurableSubsActive);
         regionBroker.setBrokerName(getBrokerName());
         regionBroker.getDestinationStatistics().setEnabled(enableStatistics);
+        regionBroker.setAllowTempAutoCreationOnSend(isAllowTempAutoCreationOnSend());
         if (brokerId != null) {
             regionBroker.setBrokerId(brokerId);
         }
@@ -1856,7 +1910,7 @@ public class BrokerService implements Service {
 
     /**
      * Strategy method to add interceptors to the broker
-     * 
+     *
      * @throws IOException
      */
     protected Broker addInterceptors(Broker broker) throws Exception {
@@ -1967,7 +2021,7 @@ public class BrokerService implements Service {
 
     /**
      * Sets hooks to be executed when broker shut down
-     * 
+     *
      * @org.apache.xbean.Property
      */
     public void setShutdownHooks(List<Runnable> hooks) throws Exception {
@@ -1975,7 +2029,7 @@ public class BrokerService implements Service {
             addShutdownHook(hook);
         }
     }
-    
+
     /**
      * Causes a clean shutdown of the container when the VM is being shut down
      */
@@ -2016,6 +2070,9 @@ public class BrokerService implements Service {
                 getBroker().addDestination(adminConnectionContext, destination,true);
             }
         }
+        if (isUseVirtualTopics()) {
+            startVirtualConsumerDestinations();
+        }
     }
 
     /**
@@ -2029,7 +2086,7 @@ public class BrokerService implements Service {
     protected void waitForSlave() {
         try {
             if (!slaveStartSignal.await(waitForSlaveTimeout, TimeUnit.MILLISECONDS)) {
-            	throw new IllegalStateException("Gave up waiting for slave to start after " + waitForSlaveTimeout + " milliseconds."); 
+                throw new IllegalStateException("Gave up waiting for slave to start after " + waitForSlaveTimeout + " milliseconds.");
             }
         } catch (InterruptedException e) {
             LOG.error("Exception waiting for slave:" + e);
@@ -2039,7 +2096,7 @@ public class BrokerService implements Service {
     protected void slaveConnectionEstablished() {
         slaveStartSignal.countDown();
     }
-    
+
     protected void startManagementContext() throws Exception {
         getManagementContext().start();
         adminView = new BrokerView(this, null);
@@ -2049,10 +2106,10 @@ public class BrokerService implements Service {
 
     /**
      * Start all transport and network connections, proxies and bridges
-     * 
+     *
      * @throws Exception
      */
-    protected void startAllConnectors() throws Exception {
+    public void startAllConnectors() throws Exception {
         if (!isSlave()) {
             Set<ActiveMQDestination> durableDestinations = getBroker().getDurableDestinations();
             List<TransportConnector> al = new ArrayList<TransportConnector>();
@@ -2174,37 +2231,49 @@ public class BrokerService implements Service {
             }
         }
     }
-    
+
     public void handleIOException(IOException exception) {
         if (ioExceptionHandler != null) {
             ioExceptionHandler.handle(exception);
          } else {
-            LOG.info("Ignoring IO exception, " + exception, exception);
+            LOG.info("No IOExceptionHandler registered, ignoring IO exception, " + exception, exception);
          }
     }
 
-    /**
-     * Starts all destiantions in persistence store. This includes all inactive
-     * destinations
-     */
-    protected void startDestinationsInPersistenceStore(Broker broker) throws Exception {
-        Set destinations = destinationFactory.getDestinations();
-        if (destinations != null) {
-            Iterator iter = destinations.iterator();
-            ConnectionContext adminConnectionContext = broker.getAdminConnectionContext();
-            if (adminConnectionContext == null) {
-                ConnectionContext context = new ConnectionContext();
-                context.setBroker(broker);
-                adminConnectionContext = context;
-                broker.setAdminConnectionContext(adminConnectionContext);
-            }
-            while (iter.hasNext()) {
-                ActiveMQDestination destination = (ActiveMQDestination) iter.next();
-                broker.addDestination(adminConnectionContext, destination,false);
+    protected void startVirtualConsumerDestinations() throws Exception {
+        ConnectionContext adminConnectionContext = getAdminConnectionContext();
+        Set<ActiveMQDestination> destinations = destinationFactory.getDestinations();
+        DestinationFilter filter = getVirtualTopicConsumerDestinationFilter();
+        if (!destinations.isEmpty()) {
+            for (ActiveMQDestination destination : destinations) {
+                if (filter.matches(destination) == true) {
+                    broker.addDestination(adminConnectionContext, destination, false);
+                }
             }
         }
     }
-    
+
+    private DestinationFilter getVirtualTopicConsumerDestinationFilter() {
+        // created at startup, so no sync needed
+        if (virtualConsumerDestinationFilter == null) {
+            Set <ActiveMQQueue> consumerDestinations = new HashSet<ActiveMQQueue>();
+            for (DestinationInterceptor interceptor : destinationInterceptors) {
+                if (interceptor instanceof VirtualDestinationInterceptor) {
+                    VirtualDestinationInterceptor virtualDestinationInterceptor = (VirtualDestinationInterceptor) interceptor;
+                    for (VirtualDestination virtualDestination: virtualDestinationInterceptor.getVirtualDestinations()) {
+                        if (virtualDestination instanceof VirtualTopic) {
+                            consumerDestinations.add(new ActiveMQQueue(((VirtualTopic) virtualDestination).getPrefix() + DestinationFilter.ANY_DESCENDENT));
+                        }
+                    }
+                }
+            }
+            ActiveMQQueue filter = new ActiveMQQueue();
+            filter.setCompositeDestinations(consumerDestinations.toArray(new ActiveMQDestination[]{}));
+            virtualConsumerDestinationFilter = DestinationFilter.parseFilter(filter);
+        }
+        return virtualConsumerDestinationFilter;
+    }
+
     protected synchronized ThreadPoolExecutor getExecutor() {
         if (this.executor == null) {
         this.executor = new ThreadPoolExecutor(1, 10, 30, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(), new ThreadFactory() {
@@ -2217,8 +2286,8 @@ public class BrokerService implements Service {
         }
         return this.executor;
     }
-    
-    protected synchronized Scheduler getScheduler() {
+
+    public synchronized Scheduler getScheduler() {
         if (this.scheduler==null) {
             this.scheduler = new Scheduler("ActiveMQ Broker["+getBrokerName()+"] Scheduler");
             try {
@@ -2289,15 +2358,15 @@ public class BrokerService implements Service {
     public void setWaitForSlave(boolean waitForSlave) {
         this.waitForSlave = waitForSlave;
     }
-  
+
     public long getWaitForSlaveTimeout() {
         return this.waitForSlaveTimeout;
     }
-    
+
     public void setWaitForSlaveTimeout(long waitForSlaveTimeout) {
         this.waitForSlaveTimeout = waitForSlaveTimeout;
     }
-    
+
     public CountDownLatch getSlaveStartSignal() {
         return slaveStartSignal;
     }
@@ -2317,12 +2386,22 @@ public class BrokerService implements Service {
     public void setPassiveSlave(boolean passiveSlave) {
         this.passiveSlave = passiveSlave;
     }
-    
+
+    /**
+     * override the Default IOException handler, called when persistence adapter
+     * has experiences File or JDBC I/O Exceptions
+     *
+     * @param ioExceptionHandler
+     */
     public void setIoExceptionHandler(IOExceptionHandler ioExceptionHandler) {
-        ioExceptionHandler.setBrokerService(this);
+        configureService(ioExceptionHandler);
         this.ioExceptionHandler = ioExceptionHandler;
     }
 
+    public IOExceptionHandler getIoExceptionHandler() {
+        return ioExceptionHandler;
+    }
+
     /**
      * @return the schedulerSupport
      */
@@ -2353,7 +2432,7 @@ public class BrokerService implements Service {
     public void setSchedulerDirectoryFile(File schedulerDirectory) {
         this.schedulerDirectoryFile = schedulerDirectory;
     }
-    
+
     public void setSchedulerDirectory(String schedulerDirectory) {
         setSchedulerDirectoryFile(new File(schedulerDirectory));
     }
@@ -2366,6 +2445,14 @@ public class BrokerService implements Service {
         this.schedulePeriodForDestinationPurge = schedulePeriodForDestinationPurge;
     }
 
+    public int getMaxPurgedDestinationsPerSweep() {
+        return this.maxPurgedDestinationsPerSweep;
+    }
+
+    public void setMaxPurgedDestinationsPerSweep(int maxPurgedDestinationsPerSweep) {
+        this.maxPurgedDestinationsPerSweep = maxPurgedDestinationsPerSweep;
+    }
+
     public BrokerContext getBrokerContext() {
         return brokerContext;
     }
@@ -2393,4 +2480,41 @@ public class BrokerService implements Service {
     public void setNetworkConnectorStartAsync(boolean networkConnectorStartAsync) {
         this.networkConnectorStartAsync = networkConnectorStartAsync;
     }
+
+    public boolean isAllowTempAutoCreationOnSend() {
+        return allowTempAutoCreationOnSend;
+    }
+
+    /**
+     * enable if temp destinations need to be propagated through a network when
+     * advisorySupport==false. This is used in conjunction with the policy
+     * gcInactiveDestinations for matching temps so they can get removed
+     * when inactive
+     *
+     * @param allowTempAutoCreationOnSend
+     */
+    public void setAllowTempAutoCreationOnSend(boolean allowTempAutoCreationOnSend) {
+        this.allowTempAutoCreationOnSend = allowTempAutoCreationOnSend;
+    }
+
+    public int getOfflineDurableSubscriberTimeout() {
+        return offlineDurableSubscriberTimeout;
+    }
+
+    public void setOfflineDurableSubscriberTimeout(int offlineDurableSubscriberTimeout) {
+        this.offlineDurableSubscriberTimeout = offlineDurableSubscriberTimeout;
+    }
+
+    public int getOfflineDurableSubscriberTaskSchedule() {
+        return offlineDurableSubscriberTaskSchedule;
+    }
+
+    public void setOfflineDurableSubscriberTaskSchedule(int offlineDurableSubscriberTaskSchedule) {
+        this.offlineDurableSubscriberTaskSchedule = offlineDurableSubscriberTaskSchedule;
+    }
+
+    public boolean shouldRecordVirtualDestination(ActiveMQDestination destination) {
+        return isUseVirtualTopics() && destination.isQueue() &&
+                getVirtualTopicConsumerDestinationFilter().matches(destination);
+    }
 }
