@@ -16,7 +16,43 @@
  */
 package org.apache.activemq.broker;
 
+import org.apache.activemq.broker.ft.MasterBroker;
+import org.apache.activemq.broker.region.ConnectionStatistics;
+import org.apache.activemq.broker.region.RegionBroker;
+import org.apache.activemq.command.*;
+import org.apache.activemq.network.DemandForwardingBridge;
+import org.apache.activemq.network.MBeanNetworkListener;
+import org.apache.activemq.network.NetworkBridgeConfiguration;
+import org.apache.activemq.network.NetworkBridgeFactory;
+import org.apache.activemq.security.MessageAuthorizationPolicy;
+import org.apache.activemq.state.CommandVisitor;
+import org.apache.activemq.state.ConnectionState;
+import org.apache.activemq.state.ConsumerState;
+import org.apache.activemq.state.ProducerState;
+import org.apache.activemq.state.SessionState;
+import org.apache.activemq.state.TransactionState;
+import org.apache.activemq.thread.DefaultThreadPools;
+import org.apache.activemq.thread.Task;
+import org.apache.activemq.thread.TaskRunner;
+import org.apache.activemq.thread.TaskRunnerFactory;
+import org.apache.activemq.transaction.Transaction;
+import org.apache.activemq.transport.DefaultTransportListener;
+import org.apache.activemq.transport.ResponseCorrelator;
+import org.apache.activemq.transport.Transport;
+import org.apache.activemq.transport.TransportDisposedIOException;
+import org.apache.activemq.transport.TransportFactory;
+import org.apache.activemq.util.IntrospectionSupport;
+import org.apache.activemq.util.MarshallingSupport;
+import org.apache.activemq.util.ServiceSupport;
+import org.apache.activemq.util.URISupport;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.slf4j.MDC;
+
+import javax.transaction.xa.XAResource;
+import java.io.EOFException;
 import java.io.IOException;
+import java.net.SocketException;
 import java.net.URI;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -33,78 +69,10 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
-import javax.management.ObjectName;
-import javax.transaction.xa.XAResource;
-
-import org.apache.activemq.broker.ft.MasterBroker;
-import org.apache.activemq.broker.region.ConnectionStatistics;
-import org.apache.activemq.broker.region.RegionBroker;
-import org.apache.activemq.command.BrokerId;
-import org.apache.activemq.command.BrokerInfo;
-import org.apache.activemq.command.Command;
-import org.apache.activemq.command.CommandTypes;
-import org.apache.activemq.command.ConnectionControl;
-import org.apache.activemq.command.ConnectionError;
-import org.apache.activemq.command.ConnectionId;
-import org.apache.activemq.command.ConnectionInfo;
-import org.apache.activemq.command.ConsumerControl;
-import org.apache.activemq.command.ConsumerId;
-import org.apache.activemq.command.ConsumerInfo;
-import org.apache.activemq.command.ControlCommand;
-import org.apache.activemq.command.DataArrayResponse;
-import org.apache.activemq.command.DestinationInfo;
-import org.apache.activemq.command.ExceptionResponse;
-import org.apache.activemq.command.FlushCommand;
-import org.apache.activemq.command.IntegerResponse;
-import org.apache.activemq.command.KeepAliveInfo;
-import org.apache.activemq.command.Message;
-import org.apache.activemq.command.MessageAck;
-import org.apache.activemq.command.MessageDispatch;
-import org.apache.activemq.command.MessageDispatchNotification;
-import org.apache.activemq.command.MessagePull;
-import org.apache.activemq.command.ProducerAck;
-import org.apache.activemq.command.ProducerId;
-import org.apache.activemq.command.ProducerInfo;
-import org.apache.activemq.command.RemoveSubscriptionInfo;
-import org.apache.activemq.command.Response;
-import org.apache.activemq.command.SessionId;
-import org.apache.activemq.command.SessionInfo;
-import org.apache.activemq.command.ShutdownInfo;
-import org.apache.activemq.command.TransactionId;
-import org.apache.activemq.command.TransactionInfo;
-import org.apache.activemq.command.WireFormatInfo;
-import org.apache.activemq.network.*;
-import org.apache.activemq.security.MessageAuthorizationPolicy;
-import org.apache.activemq.state.CommandVisitor;
-import org.apache.activemq.state.ConnectionState;
-import org.apache.activemq.state.ConsumerState;
-import org.apache.activemq.state.ProducerState;
-import org.apache.activemq.state.SessionState;
-import org.apache.activemq.state.TransactionState;
-import org.apache.activemq.thread.Task;
-import org.apache.activemq.thread.TaskRunner;
-import org.apache.activemq.thread.TaskRunnerFactory;
-import org.apache.activemq.transaction.Transaction;
-import org.apache.activemq.transport.DefaultTransportListener;
-import org.apache.activemq.transport.ResponseCorrelator;
-import org.apache.activemq.transport.Transport;
-import org.apache.activemq.transport.TransportDisposedIOException;
-import org.apache.activemq.transport.TransportFactory;
-import org.apache.activemq.util.IntrospectionSupport;
-import org.apache.activemq.util.MarshallingSupport;
-import org.apache.activemq.util.ServiceSupport;
-import org.apache.activemq.util.URISupport;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-
-import static org.apache.activemq.thread.DefaultThreadPools.getDefaultTaskRunnerFactory;
-/**
- * @version $Revision: 1.8 $
- */
 public class TransportConnection implements Connection, Task, CommandVisitor {
-    private static final Log LOG = LogFactory.getLog(TransportConnection.class);
-    private static final Log TRANSPORTLOG = LogFactory.getLog(TransportConnection.class.getName() + ".Transport");
-    private static final Log SERVICELOG = LogFactory.getLog(TransportConnection.class.getName() + ".Service");
+    private static final Logger LOG = LoggerFactory.getLogger(TransportConnection.class);
+    private static final Logger TRANSPORTLOG = LoggerFactory.getLogger(TransportConnection.class.getName() + ".Transport");
+    private static final Logger SERVICELOG = LoggerFactory.getLogger(TransportConnection.class.getName() + ".Service");
     // Keeps track of the broker and connector that created this connection.
     protected final Broker broker;
     protected final TransportConnector connector;
@@ -151,17 +119,14 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     private TransportConnectionStateRegister connectionStateRegister = new SingleTransportConnectionStateRegister();
     private final ReentrantReadWriteLock serviceLock = new ReentrantReadWriteLock();
     private String duplexNetworkConnectorId;
+    private Throwable stopError = null;
 
     /**
-     * @param connector
-     * @param transport
-     * @param broker
-     * @param taskRunnerFactory
-     *            - can be null if you want direct dispatch to the transport
-     *            else commands are sent async.
+     * @param taskRunnerFactory - can be null if you want direct dispatch to the transport
+     *                          else commands are sent async.
      */
     public TransportConnection(TransportConnector connector, final Transport transport, Broker broker,
-            TaskRunnerFactory taskRunnerFactory) {
+                               TaskRunnerFactory taskRunnerFactory) {
         this.connector = connector;
         this.broker = broker;
         this.messageAuthorizationPolicy = connector.getMessageAuthorizationPolicy();
@@ -232,20 +197,28 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         if (!stopping.get()) {
             transportException.set(e);
             if (TRANSPORTLOG.isDebugEnabled()) {
-                TRANSPORTLOG.debug("Transport failed: " + e, e);
-            } else if (TRANSPORTLOG.isInfoEnabled()) {
-                TRANSPORTLOG.info("Transport failed: " + e);
+                TRANSPORTLOG.debug(this + " failed: " + e, e);
+            } else if (TRANSPORTLOG.isWarnEnabled() && !expected(e)) {
+                TRANSPORTLOG.warn(this + " failed: " + e);
             }
             stopAsync();
         }
     }
 
+    private boolean expected(IOException e) {
+        return isStomp() &&
+                ((e instanceof SocketException && e.getMessage().indexOf("reset") != -1) || e instanceof EOFException);
+    }
+
+    private boolean isStomp() {
+        URI uri = connector.getUri();
+        return uri != null && uri.getScheme() != null && uri.getScheme().indexOf("stomp") != -1;
+    }
+
     /**
      * Calls the serviceException method in an async thread. Since handling a
      * service exception closes a socket, we should not tie up broker threads
      * since client sockets may hang or cause deadlocks.
-     * 
-     * @param e
      */
     public void serviceExceptionAsync(final IOException e) {
         if (asyncException.compareAndSet(false, true)) {
@@ -278,6 +251,8 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 ConnectionError ce = new ConnectionError();
                 ce.setException(e);
                 dispatchSync(ce);
+                // Record the error that caused the transport to stop
+                this.stopError = e;
                 // Wait a little bit to try to get the output buffer to flush
                 // the exption notification to the client.
                 try {
@@ -295,7 +270,11 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 SERVICELOG.warn("Async error occurred: " + e, e);
                 ConnectionError ce = new ConnectionError();
                 ce.setException(e);
-                dispatchAsync(ce);
+                if (pendingStop) {
+                    dispatchSync(ce);
+                } else {
+                    dispatchAsync(ce);
+                }
             } finally {
                 inServiceException = false;
             }
@@ -303,16 +282,27 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     public Response service(Command command) {
+        MDC.put("activemq.connector", connector.getUri().toString());
         Response response = null;
         boolean responseRequired = command.isResponseRequired();
         int commandId = command.getCommandId();
         try {
-            response = command.visit(this);
+            if (!pendingStop) {
+                response = command.visit(this);
+            } else {
+                response = new ExceptionResponse(this.stopError);
+            }
         } catch (Throwable e) {
             if (SERVICELOG.isDebugEnabled() && e.getClass() != BrokerStoppedException.class) {
                 SERVICELOG.debug("Error occured while processing " + (responseRequired ? "sync" : "async")
                         + " command: " + command + ", exception: " + e, e);
             }
+
+            if(e instanceof java.lang.SecurityException){
+                // still need to close this down - in case the peer of this transport doesn't play nice
+                delayedStop(2000, "Failed with SecurityException: " + e.getLocalizedMessage(), e);
+            }
+
             if (responseRequired) {
                 response = new ExceptionResponse(e);
             } else {
@@ -334,6 +324,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             }
             context = null;
         }
+        MDC.remove("activemq.connector");
         return response;
     }
 
@@ -465,7 +456,9 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
 
     public Response processMessageAck(MessageAck ack) throws Exception {
         ConsumerBrokerExchange consumerExchange = getConsumerBrokerExchange(ack.getConsumerId());
-        broker.acknowledge(consumerExchange, ack);
+        if (consumerExchange != null) {
+            broker.acknowledge(consumerExchange, ack);
+        }
         return null;
     }
 
@@ -500,6 +493,10 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         SessionId sessionId = info.getProducerId().getParentId();
         ConnectionId connectionId = sessionId.getParentId();
         TransportConnectionState cs = lookupConnectionState(connectionId);
+        if (cs == null) {
+            throw new IllegalStateException("Cannot add a producer to a connection that had not been registered: "
+                    + connectionId);
+        }
         SessionState ss = cs.getSessionState(sessionId);
         if (ss == null) {
             throw new IllegalStateException("Cannot add a producer to a session that had not been registered: "
@@ -539,6 +536,10 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         SessionId sessionId = info.getConsumerId().getParentId();
         ConnectionId connectionId = sessionId.getParentId();
         TransportConnectionState cs = lookupConnectionState(connectionId);
+        if (cs == null) {
+            throw new IllegalStateException("Cannot add a consumer to a connection that had not been registered: "
+                    + connectionId);
+        }
         SessionState ss = cs.getSessionState(sessionId);
         if (ss == null) {
             throw new IllegalStateException(broker.getBrokerName()
@@ -549,6 +550,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             broker.addConsumer(cs.getContext(), info);
             try {
                 ss.addConsumer(info);
+                addConsumerBrokerExchange(info.getConsumerId());
             } catch (IllegalStateException e) {
                 broker.removeConsumer(cs.getContext(), info);
             }
@@ -610,16 +612,14 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         // this down.
         session.shutdown();
         // Cascade the connection stop to the consumers and producers.
-        for (Iterator iter = session.getConsumerIds().iterator(); iter.hasNext();) {
-            ConsumerId consumerId = (ConsumerId) iter.next();
+        for (ConsumerId consumerId : session.getConsumerIds()) {
             try {
                 processRemoveConsumer(consumerId, lastDeliveredSequenceId);
             } catch (Throwable e) {
                 LOG.warn("Failed to remove consumer: " + consumerId + ". Reason: " + e, e);
             }
         }
-        for (Iterator iter = session.getProducerIds().iterator(); iter.hasNext();) {
-            ProducerId producerId = (ProducerId) iter.next();
+        for (ProducerId producerId : session.getProducerIds()) {
             try {
                 processRemoveProducer(producerId);
             } catch (Throwable e) {
@@ -669,8 +669,8 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             }
         }
         registerConnectionState(info.getConnectionId(), state);
-        LOG.debug("Setting up new connection id: " + info.getConnectionId() + ", address: " + getRemoteAddress());
-        this.faultTolerantConnection=info.isFaultTolerant();
+        LOG.debug("Setting up new connection id: " + info.getConnectionId() + ", address: " + getRemoteAddress() + ", info: " + info);
+        this.faultTolerantConnection = info.isFaultTolerant();
         // Setup the context.
         String clientId = info.getClientId();
         context = new ConnectionContext();
@@ -688,8 +688,12 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         context.setWireFormatInfo(wireFormatInfo);
         context.setReconnect(info.isFailoverReconnect());
         this.manageable = info.isManageable();
+        context.setConnectionState(state);
         state.setContext(context);
         state.setConnection(this);
+        if (info.getClientIp() == null) {
+            info.setClientIp(getRemoteAddress());
+        }
        
         try {
             broker.addConnection(context, info);
@@ -698,7 +702,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 brokerConnectionStates.remove(info.getConnectionId());
             }
             unregisterConnectionState(info.getConnectionId());
-            LOG.warn("Failed to add Connection " + info.getConnectionId() + ", reason: " +  e.toString());
+            LOG.warn("Failed to add Connection " + info.getConnectionId() + ", reason: " + e.toString());
             if (LOG.isDebugEnabled()) {
                 LOG.debug("Exception detail:", e);
             }
@@ -708,6 +712,9 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             // send ConnectionCommand
             ConnectionControl command = this.connector.getConnectionControl();
             command.setFaultTolerant(broker.isFaultTolerantConfiguration());
+            if (info.isFailoverReconnect()) {
+                command.setRebalanceConnection(false);
+            }
             dispatchAsync(command);
         }
         return null;
@@ -719,12 +726,10 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         TransportConnectionState cs = lookupConnectionState(id);
         if (cs != null) {
             // Don't allow things to be added to the connection state while we
-            // are
-            // shutting down.
+            // are shutting down.
             cs.shutdown();
             // Cascade the connection stop to the sessions.
-            for (Iterator iter = cs.getSessionIds().iterator(); iter.hasNext();) {
-                SessionId sessionId = (SessionId) iter.next();
+            for (SessionId sessionId : cs.getSessionIds()) {
                 try {
                     processRemoveSession(sessionId, lastDeliveredSequenceId);
                 } catch (Throwable e) {
@@ -732,8 +737,8 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 }
             }
             // Cascade the connection stop to temp destinations.
-            for (Iterator iter = cs.getTempDesinations().iterator(); iter.hasNext();) {
-                DestinationInfo di = (DestinationInfo) iter.next();
+            for (Iterator<DestinationInfo> iter = cs.getTempDestinations().iterator(); iter.hasNext(); ) {
+                DestinationInfo di = iter.next();
                 try {
                     broker.removeDestination(cs.getContext(), di.getDestination(), 0);
                 } catch (Throwable e) {
@@ -773,7 +778,6 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     public void dispatchSync(Command message) {
-        // getStatistics().getEnqueues().increment();
         try {
             processDispatch(message);
         } catch (IOException e) {
@@ -783,7 +787,6 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
 
     public void dispatchAsync(Command message) {
         if (!stopping.get()) {
-            // getStatistics().getEnqueues().increment();
             if (taskRunner == null) {
                 dispatchSync(message);
             } else {
@@ -825,13 +828,12 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                     sub.run();
                 }
             }
-            // getStatistics().getDequeues().increment();
         }
     }
 
     public boolean iterate() {
         try {
-            if (stopping.get()) {
+            if (pendingStop || stopping.get()) {
                 if (dispatchStopped.compareAndSet(false, true)) {
                     if (transportException.get() == null) {
                         try {
@@ -884,9 +886,9 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     public void start() throws Exception {
-        starting = true;
         try {
             synchronized (this) {
+                starting = true;
                 if (taskRunnerFactory != null) {
                     taskRunner = taskRunnerFactory.createTaskRunner(this, "ActiveMQ Connection Dispatcher: "
                             + getRemoteAddress());
@@ -907,53 +909,77 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
             }
         } catch (Exception e) {
             // Force clean up on an error starting up.
-            stop();
+            pendingStop = true;
             throw e;
         } finally {
             // stop() can be called from within the above block,
             // but we want to be sure start() completes before
             // stop() runs, so queue the stop until right now:
-            starting = false;
-            if (pendingStop) {
-                LOG.debug("Calling the delayed stop()");
+            setStarting(false);
+            if (isPendingStop()) {
+                LOG.debug("Calling the delayed stop() after start() " + this);
                 stop();
             }
         }
     }
 
     public void stop() throws Exception {
-        synchronized (this) {
-            pendingStop = true;
-            if (starting) {
-                LOG.debug("stop() called in the middle of start(). Delaying...");
-                return;
-            }
-        }
         stopAsync();
         while (!stopped.await(5, TimeUnit.SECONDS)) {
             LOG.info("The connection to '" + transport.getRemoteAddress() + "' is taking a long time to shutdown.");
         }
     }
 
+    public void delayedStop(final int waitTime, final String reason, Throwable cause) {
+        if (waitTime > 0) {
+            synchronized (this) {
+                pendingStop = true;
+                stopError = cause;
+            }
+            try {
+                DefaultThreadPools.getDefaultTaskRunnerFactory().execute(new Runnable() {
+                    public void run() {
+                        try {
+                            Thread.sleep(waitTime);
+                            stopAsync();
+                            LOG.info("Stopping " + transport.getRemoteAddress() + " because " + reason);
+                        } catch (InterruptedException e) {
+                        }
+                    }
+                }, "delayedStop:" + transport.getRemoteAddress());
+            } catch (Throwable t) {
+                LOG.warn("cannot create stopAsync :", t);
+            }
+        }
+    }
+
     public void stopAsync() {
-        // If we're in the middle of starting
-        // then go no further... for now.
+        // If we're in the middle of starting then go no further... for now.
+        synchronized (this) {
+            pendingStop = true;
+            if (starting) {
+                LOG.debug("stopAsync() called in the middle of start(). Delaying till start completes..");
+                return;
+            }
+        }
         if (stopping.compareAndSet(false, true)) {
             // Let all the connection contexts know we are shutting down
             // so that in progress operations can notice and unblock.
             List<TransportConnectionState> connectionStates = listConnectionStates();
             for (TransportConnectionState cs : connectionStates) {
-                cs.getContext().getStopping().set(true);
+                ConnectionContext connectionContext = cs.getContext();
+                if (connectionContext != null) {
+                    connectionContext.getStopping().set(true);
+                }
             }
             try {
-                getDefaultTaskRunnerFactory().execute(new Runnable(){
+                DefaultThreadPools.getDefaultTaskRunnerFactory().execute(new Runnable() {
                     public void run() {
                         serviceLock.writeLock().lock();
                         try {
                             doStop();
                         } catch (Throwable e) {
-                            LOG.debug("Error occured while shutting down a connection to '" + transport.getRemoteAddress()
-                                    + "': ", e);
+                            LOG.debug("Error occurred while shutting down a connection " + this, e);
                         } finally {
                             stopped.countDown();
                             serviceLock.writeLock().unlock();
@@ -1000,7 +1026,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         // Run the MessageDispatch callbacks so that message references get
         // cleaned up.
         synchronized (dispatchQueue) {
-            for (Iterator<Command> iter = dispatchQueue.iterator(); iter.hasNext();) {
+            for (Iterator<Command> iter = dispatchQueue.iterator(); iter.hasNext(); ) {
                 Command command = iter.next();
                 if (command.isMessageDispatch()) {
                     MessageDispatch md = (MessageDispatch) command;
@@ -1040,8 +1066,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     /**
-     * @param blockedCandidate
-     *            The blockedCandidate to set.
+     * @param blockedCandidate The blockedCandidate to set.
      */
     public void setBlockedCandidate(boolean blockedCandidate) {
         this.blockedCandidate = blockedCandidate;
@@ -1055,8 +1080,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     /**
-     * @param markedCandidate
-     *            The markedCandidate to set.
+     * @param markedCandidate The markedCandidate to set.
      */
     public void setMarkedCandidate(boolean markedCandidate) {
         this.markedCandidate = markedCandidate;
@@ -1067,8 +1091,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     /**
-     * @param slow
-     *            The slow to set.
+     * @param slow The slow to set.
      */
     public void setSlow(boolean slow) {
         this.slow = slow;
@@ -1112,16 +1135,14 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     /**
-     * @param blocked
-     *            The blocked to set.
+     * @param blocked The blocked to set.
      */
     public void setBlocked(boolean blocked) {
         this.blocked = blocked;
     }
 
     /**
-     * @param connected
-     *            The connected to set.
+     * @param connected The connected to set.
      */
     public void setConnected(boolean connected) {
         this.connected = connected;
@@ -1135,8 +1156,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     /**
-     * @param active
-     *            The active to set.
+     * @param active The active to set.
      */
     public void setActive(boolean active) {
         this.active = active;
@@ -1154,7 +1174,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
     
     public boolean isFaultTolerantConnection() {
-       return this.faultTolerantConnection;
+        return this.faultTolerantConnection;
     }
 
     protected synchronized void setStarting(boolean starting) {
@@ -1186,7 +1206,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 masterBroker = new MasterBroker(parent, transport);
                 masterBroker.startProcessing();
             }
-            LOG.info((passive?"Passive":"Active")+" Slave Broker " + info.getBrokerName() + " is attached");
+            LOG.info((passive ? "Passive" : "Active") + " Slave Broker " + info.getBrokerName() + " is attached");
             bService.slaveConnectionEstablished();
         } else if (info.isNetworkConnection() && info.isDuplexConnection()) {
             // so this TransportConnection is the rear end of a network bridge
@@ -1207,7 +1227,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 String duplexNetworkConnectorId = config.getName() + "@" + info.getBrokerId(); 
                 CopyOnWriteArrayList<TransportConnection> connections = this.connector.getConnections();
                 synchronized (connections) {
-                    for (Iterator<TransportConnection> iter = connections.iterator(); iter.hasNext();) {
+                    for (Iterator<TransportConnection> iter = connections.iterator(); iter.hasNext(); ) {
                         TransportConnection c = iter.next();
                         if ((c != this) && (duplexNetworkConnectorId.equals(c.getDuplexNetworkConnectorId()))) {
                             LOG.warn("Stopping an existing active duplex connection [" + c + "] for network connector (" + duplexNetworkConnectorId + ").");
@@ -1243,7 +1263,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 LOG.warn("Duplex bridge " + duplexNetworkConnectorId + " was stopped before it was correctly started.");
                 return null;
             } catch (Exception e) {
-                LOG.error("Failed to create responder end of duplex network bridge " + duplexNetworkConnectorId , e);
+                LOG.error("Failed to create responder end of duplex network bridge " + duplexNetworkConnectorId, e);
                 return null;
             }
         }
@@ -1260,7 +1280,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
         return null;
     }
 
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({ "unchecked", "rawtypes" })
     private HashMap<String, String> createMap(Properties properties) {
         return new HashMap(properties);
     }
@@ -1303,10 +1323,10 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
                 result = new ProducerBrokerExchange();
                 TransportConnectionState state = lookupConnectionState(id);              
                 context = state.getContext();
-                if (context.isReconnect()) {
+                result.setConnectionContext(context);
+                if (context.isReconnect() || (context.isNetworkConnection() && connector.isAuditNetworkProducers())) {
                     result.setLastStoredSequenceId(broker.getBrokerService().getPersistenceAdapter().getLastProducerSequenceId(id));
                 }
-                result.setConnectionContext(context);
                 SessionState ss = state.getSessionState(id.getParentId());
                 if (ss != null) {
                     result.setProducerState(ss.getProducerState(id));
@@ -1331,6 +1351,11 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     private ConsumerBrokerExchange getConsumerBrokerExchange(ConsumerId id) {
+        ConsumerBrokerExchange result = consumerExchanges.get(id);
+        return result;
+    }
+
+    private ConsumerBrokerExchange addConsumerBrokerExchange(ConsumerId id) {
         ConsumerBrokerExchange result = consumerExchanges.get(id);
         if (result == null) {
             synchronized (consumerExchanges) {
@@ -1396,7 +1421,7 @@ public class TransportConnection implements Connection, Task, CommandVisitor {
     }
 
     protected synchronized TransportConnectionState registerConnectionState(ConnectionId connectionId,
-            TransportConnectionState state) {
+                                                                            TransportConnectionState state) {
         TransportConnectionState cs = null;
         if (!connectionStateRegister.isEmpty() && !connectionStateRegister.doesHandleMultipleConnectionStates()) {
             // swap implementations
