@@ -41,6 +41,7 @@ import javax.management.openmbean.TabularType;
 import org.apache.activemq.broker.Broker;
 import org.apache.activemq.broker.BrokerService;
 import org.apache.activemq.broker.ConnectionContext;
+import org.apache.activemq.broker.ProducerBrokerExchange;
 import org.apache.activemq.broker.jmx.OpenTypeSupport.OpenTypeFactory;
 import org.apache.activemq.broker.region.Destination;
 import org.apache.activemq.broker.region.DestinationFactory;
@@ -54,28 +55,29 @@ import org.apache.activemq.broker.region.Topic;
 import org.apache.activemq.broker.region.TopicRegion;
 import org.apache.activemq.broker.region.TopicSubscription;
 import org.apache.activemq.broker.region.policy.AbortSlowConsumerStrategy;
-import org.apache.activemq.broker.region.policy.SlowConsumerStrategy;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQMessage;
 import org.apache.activemq.command.ActiveMQTopic;
 import org.apache.activemq.command.ConsumerInfo;
 import org.apache.activemq.command.Message;
 import org.apache.activemq.command.MessageId;
+import org.apache.activemq.command.ProducerInfo;
 import org.apache.activemq.command.SubscriptionInfo;
 import org.apache.activemq.store.MessageRecoveryListener;
 import org.apache.activemq.store.PersistenceAdapter;
 import org.apache.activemq.store.TopicMessageStore;
 import org.apache.activemq.thread.Scheduler;
 import org.apache.activemq.thread.TaskRunnerFactory;
+import org.apache.activemq.transaction.XATransaction;
 import org.apache.activemq.usage.SystemUsage;
 import org.apache.activemq.util.JMXSupport;
 import org.apache.activemq.util.ServiceStopper;
 import org.apache.activemq.util.SubscriptionKey;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public class ManagedRegionBroker extends RegionBroker {
-    private static final Log LOG = LogFactory.getLog(ManagedRegionBroker.class);
+    private static final Logger LOG = LoggerFactory.getLogger(ManagedRegionBroker.class);
     private final ManagementContext managementContext;
     private final ObjectName brokerObjectName;
     private final Map<ObjectName, DestinationView> topics = new ConcurrentHashMap<ObjectName, DestinationView>();
@@ -88,6 +90,11 @@ public class ManagedRegionBroker extends RegionBroker {
     private final Map<ObjectName, SubscriptionView> inactiveDurableTopicSubscribers = new ConcurrentHashMap<ObjectName, SubscriptionView>();
     private final Map<ObjectName, SubscriptionView> temporaryQueueSubscribers = new ConcurrentHashMap<ObjectName, SubscriptionView>();
     private final Map<ObjectName, SubscriptionView> temporaryTopicSubscribers = new ConcurrentHashMap<ObjectName, SubscriptionView>();
+    private final Map<ObjectName, ProducerView> queueProducers = new ConcurrentHashMap<ObjectName, ProducerView>();
+    private final Map<ObjectName, ProducerView> topicProducers = new ConcurrentHashMap<ObjectName, ProducerView>();
+    private final Map<ObjectName, ProducerView> temporaryQueueProducers = new ConcurrentHashMap<ObjectName, ProducerView>();
+    private final Map<ObjectName, ProducerView> temporaryTopicProducers = new ConcurrentHashMap<ObjectName, ProducerView>();
+    private final Map<ObjectName, ProducerView> dynamicDestinationProducers = new ConcurrentHashMap<ObjectName, ProducerView>();
     private final Map<SubscriptionKey, ObjectName> subscriptionKeys = new ConcurrentHashMap<SubscriptionKey, ObjectName>();
     private final Map<Subscription, ObjectName> subscriptionMap = new ConcurrentHashMap<Subscription, ObjectName>();
     private final Set<ObjectName> registeredMBeans = new CopyOnWriteArraySet<ObjectName>();
@@ -178,7 +185,7 @@ public class ManagedRegionBroker extends RegionBroker {
     public ObjectName registerSubscription(ConnectionContext context, Subscription sub) {
         String connectionClientId = context.getClientId();
         ObjectName brokerJmxObjectName = brokerObjectName;
-        String objectNameStr = getSubscriptionObjectName(sub, connectionClientId, brokerJmxObjectName);
+        String objectNameStr = getSubscriptionObjectName(sub.getConsumerInfo(), connectionClientId, brokerJmxObjectName);
         SubscriptionKey key = new SubscriptionKey(context.getClientId(), sub.getConsumerInfo().getSubscriptionName());
         try {
             ObjectName objectName = new ObjectName(objectNameStr);
@@ -189,7 +196,8 @@ public class ManagedRegionBroker extends RegionBroker {
                 info.setClientId(context.getClientId());
                 info.setSubscriptionName(sub.getConsumerInfo().getSubscriptionName());
                 info.setDestination(sub.getConsumerInfo().getDestination());
-                addInactiveSubscription(key, info);
+                info.setSelector(sub.getSelector());
+                addInactiveSubscription(key, info, sub);
             } else {
                 if (sub.getConsumerInfo().isDurable()) {
                     view = new DurableSubscriptionView(this, context.getClientId(), sub);
@@ -210,21 +218,21 @@ public class ManagedRegionBroker extends RegionBroker {
         }
     }
 
-    public static String getSubscriptionObjectName(Subscription sub, String connectionClientId, ObjectName brokerJmxObjectName) {
+    public static String getSubscriptionObjectName(ConsumerInfo info, String connectionClientId, ObjectName brokerJmxObjectName) {
         Hashtable map = brokerJmxObjectName.getKeyPropertyList();
         String brokerDomain = brokerJmxObjectName.getDomain();
         String objectNameStr = brokerDomain + ":" + "BrokerName=" + map.get("BrokerName") + ",Type=Subscription,";
-        String destinationType = "destinationType=" + sub.getConsumerInfo().getDestination().getDestinationTypeAsString();
-        String destinationName = "destinationName=" + JMXSupport.encodeObjectNamePart(sub.getConsumerInfo().getDestination().getPhysicalName());
+        String destinationType = "destinationType=" + info.getDestination().getDestinationTypeAsString();
+        String destinationName = "destinationName=" + JMXSupport.encodeObjectNamePart(info.getDestination().getPhysicalName());
         String clientId = "clientId=" + JMXSupport.encodeObjectNamePart(connectionClientId);
         String persistentMode = "persistentMode=";
         String consumerId = "";
-        if (sub.getConsumerInfo().isDurable()) {
-            persistentMode += "Durable,subscriptionID=" + JMXSupport.encodeObjectNamePart(sub.getConsumerInfo().getSubscriptionName());
+        if (info.isDurable()) {
+            persistentMode += "Durable,subscriptionID=" + JMXSupport.encodeObjectNamePart(info.getSubscriptionName());
         } else {
             persistentMode += "Non-Durable";
-            if (sub.getConsumerInfo() != null && sub.getConsumerInfo().getConsumerId() != null) {
-                consumerId = ",consumerId=" + JMXSupport.encodeObjectNamePart(sub.getConsumerInfo().getConsumerId().toString());
+            if (info.getConsumerId() != null) {
+                consumerId = ",consumerId=" + JMXSupport.encodeObjectNamePart(info.getConsumerId().toString());
             }
         }
         objectNameStr += persistentMode + ",";
@@ -258,6 +266,41 @@ public class ManagedRegionBroker extends RegionBroker {
         super.removeConsumer(context, info);
     }
 
+    @Override
+    public void addProducer(ConnectionContext context, ProducerInfo info)
+            throws Exception {
+        super.addProducer(context, info);
+        String connectionClientId = context.getClientId();
+        ObjectName objectName = createObjectName(info, connectionClientId);
+        ProducerView view = new ProducerView(info, connectionClientId, this);
+        registerProducer(objectName, info.getDestination(), view);
+    }
+
+    @Override
+    public void removeProducer(ConnectionContext context, ProducerInfo info) throws Exception {
+        ObjectName objectName = createObjectName(info, context.getClientId());
+        unregisterProducer(objectName);
+        super.removeProducer(context, info);
+    }
+
+    @Override
+    public void send(ProducerBrokerExchange exchange, Message message) throws Exception {
+        if (exchange != null && exchange.getProducerState() != null && exchange.getProducerState().getInfo() != null) {
+            ProducerInfo info = exchange.getProducerState().getInfo();
+            if (info.getDestination() == null && info.getProducerId() != null) {
+                ObjectName objectName = createObjectName(info, exchange.getConnectionContext().getClientId());
+                ProducerView view = this.dynamicDestinationProducers.get(objectName);
+                if (view != null) {
+                    ActiveMQDestination dest = message.getDestination();
+                    if (dest != null) {
+                        view.setLastUsedDestinationName(dest);
+                    }
+                }
+            }
+         }
+        super.send(exchange, message);
+    }
+
     public void unregisterSubscription(Subscription sub) {
         ObjectName name = subscriptionMap.remove(sub);
         if (name != null) {
@@ -325,6 +368,51 @@ public class ManagedRegionBroker extends RegionBroker {
         }
     }
 
+    protected void registerProducer(ObjectName key, ActiveMQDestination dest, ProducerView view) throws Exception {
+
+        if (dest != null) {
+            if (dest.isQueue()) {
+                if (dest.isTemporary()) {
+                    temporaryQueueProducers.put(key, view);
+                } else {
+                    queueProducers.put(key, view);
+                }
+            } else {
+                if (dest.isTemporary()) {
+                    temporaryTopicProducers.put(key, view);
+                } else {
+                    topicProducers.put(key, view);
+                }
+            }
+        } else {
+            dynamicDestinationProducers.put(key, view);
+        }
+
+        try {
+            AnnotatedMBean.registerMBean(managementContext, view, key);
+            registeredMBeans.add(key);
+        } catch (Throwable e) {
+            LOG.warn("Failed to register MBean: " + key);
+            LOG.debug("Failure reason: " + e, e);
+        }
+    }
+
+    protected void unregisterProducer(ObjectName key) throws Exception {
+        queueProducers.remove(key);
+        topicProducers.remove(key);
+        temporaryQueueProducers.remove(key);
+        temporaryTopicProducers.remove(key);
+        dynamicDestinationProducers.remove(key);
+        if (registeredMBeans.remove(key)) {
+            try {
+                managementContext.unregisterMBean(key);
+            } catch (Throwable e) {
+                LOG.warn("Failed to unregister MBean: " + key);
+                LOG.debug("Failure reason: " + e, e);
+            }
+        }
+    }
+
     private void removeAndRemember(Map<ObjectName, DestinationView> map, ObjectName key, DestinationView view) {
         DestinationView candidate = map.remove(key);
         if (candidate != null && view == null) {
@@ -395,7 +483,8 @@ public class ManagedRegionBroker extends RegionBroker {
                 info.setClientId(subscriptionKey.getClientId());
                 info.setSubscriptionName(subscriptionKey.getSubscriptionName());
                 info.setDestination(new ActiveMQTopic(view.getDestinationName()));
-                addInactiveSubscription(subscriptionKey, info);
+                info.setSelector(view.getSelector());
+                addInactiveSubscription(subscriptionKey, info, (brokerService.isKeepDurableSubsActive() ? view.subscription : null));
             }
         }
     }
@@ -406,7 +495,7 @@ public class ManagedRegionBroker extends RegionBroker {
         if (destinations != null) {
             for (Iterator iter = destinations.iterator(); iter.hasNext();) {
                 ActiveMQDestination dest = (ActiveMQDestination)iter.next();
-                if (dest.isTopic()) {                
+                if (dest.isTopic()) {
                     SubscriptionInfo[] infos = destinationFactory.getAllDurableSubscriptions((ActiveMQTopic)dest);
                     if (infos != null) {
                         for (int i = 0; i < infos.length; i++) {
@@ -425,7 +514,7 @@ public class ManagedRegionBroker extends RegionBroker {
             Map.Entry entry = (Entry)i.next();
             SubscriptionKey key = (SubscriptionKey)entry.getKey();
             SubscriptionInfo info = (SubscriptionInfo)entry.getValue();
-            addInactiveSubscription(key, info);
+            addInactiveSubscription(key, info, null);
         }
     }
 
@@ -438,12 +527,11 @@ public class ManagedRegionBroker extends RegionBroker {
         return known;
     }
 
-    protected void addInactiveSubscription(SubscriptionKey key, SubscriptionInfo info) {
-        Hashtable map = brokerObjectName.getKeyPropertyList();
+    protected void addInactiveSubscription(SubscriptionKey key, SubscriptionInfo info, Subscription subscription) {
         try {
-            ObjectName objectName = new ObjectName(brokerObjectName.getDomain() + ":" + "BrokerName=" + map.get("BrokerName") + "," + "Type=Subscription," + "active=false,"
-                                                   + "name=" + JMXSupport.encodeObjectNamePart(key.toString()) + "");
-            SubscriptionView view = new InactiveDurableSubscriptionView(this, key.getClientId(), info);
+            ConsumerInfo offlineConsumerInfo = subscription != null ? subscription.getConsumerInfo() : ((TopicRegion)getTopicRegion()).createInactiveConsumerInfo(info);
+            ObjectName objectName = new ObjectName(getSubscriptionObjectName(offlineConsumerInfo, info.getClientId(), brokerObjectName));
+            SubscriptionView view = new InactiveDurableSubscriptionView(this, key.getClientId(), info, subscription);
 
             try {
                 AnnotatedMBean.registerMBean(managementContext, view, objectName);
@@ -508,7 +596,7 @@ public class ManagedRegionBroker extends RegionBroker {
                 public boolean hasSpace() {
                     return true;
                 }
-                
+
                 public boolean isDuplicate(MessageId id) {
                     return false;
                 }
@@ -570,6 +658,31 @@ public class ManagedRegionBroker extends RegionBroker {
         return set.toArray(new ObjectName[set.size()]);
     }
 
+    protected ObjectName[] getTopicProducers() {
+        Set<ObjectName> set = topicProducers.keySet();
+        return set.toArray(new ObjectName[set.size()]);
+    }
+
+    protected ObjectName[] getQueueProducers() {
+        Set<ObjectName> set = queueProducers.keySet();
+        return set.toArray(new ObjectName[set.size()]);
+    }
+
+    protected ObjectName[] getTemporaryTopicProducers() {
+        Set<ObjectName> set = temporaryTopicProducers.keySet();
+        return set.toArray(new ObjectName[set.size()]);
+    }
+
+    protected ObjectName[] getTemporaryQueueProducers() {
+        Set<ObjectName> set = temporaryQueueProducers.keySet();
+        return set.toArray(new ObjectName[set.size()]);
+    }
+
+    protected ObjectName[] getDynamicDestinationProducers() {
+        Set<ObjectName> set = dynamicDestinationProducers.keySet();
+        return set.toArray(new ObjectName[set.size()]);
+    }
+
     public Broker getContextBroker() {
         return contextBroker;
     }
@@ -587,6 +700,32 @@ public class ManagedRegionBroker extends RegionBroker {
         return objectName;
     }
 
+    protected ObjectName createObjectName(ProducerInfo producerInfo, String connectionClientId) throws MalformedObjectNameException {
+        // Build the object name for the producer info
+        Hashtable map = brokerObjectName.getKeyPropertyList();
+
+        String destinationType = "destinationType=";
+        String destinationName = "destinationName=";
+
+        if (producerInfo.getDestination() == null) {
+            destinationType += "Dynamic";
+            destinationName = null;
+        } else {
+            destinationType += producerInfo.getDestination().getDestinationTypeAsString();
+            destinationName += JMXSupport.encodeObjectNamePart(producerInfo.getDestination().getPhysicalName());
+        }
+
+        String clientId = "clientId=" + JMXSupport.encodeObjectNamePart(connectionClientId);
+        String producerId = "producerId=" + JMXSupport.encodeObjectNamePart(producerInfo.getProducerId().toString());
+
+        ObjectName objectName = new ObjectName(brokerObjectName.getDomain() + ":" + "BrokerName=" + map.get("BrokerName") + ","
+                                               + "Type=Producer" + ","
+                                               + destinationType + ","
+                                               + (destinationName != null ? destinationName + "," : "")
+                                               + clientId + "," + producerId);
+        return objectName;
+    }
+
     public ObjectName registerSlowConsumerStrategy(AbortSlowConsumerStrategy strategy) throws MalformedObjectNameException {
         ObjectName objectName = null;
         try {
@@ -603,11 +742,50 @@ public class ManagedRegionBroker extends RegionBroker {
         return objectName;
     }
 
+    protected ObjectName createObjectName(XATransaction transaction) throws MalformedObjectNameException {
+        Hashtable map = brokerObjectName.getKeyPropertyList();
+        ObjectName objectName = new ObjectName(brokerObjectName.getDomain() + ":" + "BrokerName=" + map.get("BrokerName")
+                                               + "," + "Type=RecoveredXaTransaction"
+                                               + "," + "Xid="
+                                               + JMXSupport.encodeObjectNamePart(transaction.getTransactionId().toString()));
+        return objectName;
+    }
+
+    public void registerRecoveredTransactionMBean(XATransaction transaction) {
+        try {
+            ObjectName objectName = createObjectName(transaction);
+            if (!registeredMBeans.contains(objectName))  {
+                RecoveredXATransactionView view = new RecoveredXATransactionView(this, transaction);
+                AnnotatedMBean.registerMBean(managementContext, view, objectName);
+                registeredMBeans.add(objectName);
+            }
+        } catch (Exception e) {
+            LOG.warn("Failed to register prepared transaction MBean: " + transaction);
+            LOG.debug("Failure reason: " + e, e);
+        }
+    }
+
+    public void unregister(XATransaction transaction) {
+        try {
+            ObjectName objectName = createObjectName(transaction);
+            if (registeredMBeans.remove(objectName)) {
+                try {
+                    managementContext.unregisterMBean(objectName);
+                } catch (Throwable e) {
+                    LOG.warn("Failed to unregister MBean: " + objectName);
+                    LOG.debug("Failure reason: " + e, e);
+                }
+            }
+        } catch (Exception e) {
+            LOG.warn("Failed to create object name to unregister " + transaction, e);
+        }
+    }
+
     private ObjectName createObjectName(AbortSlowConsumerStrategy strategy) throws MalformedObjectNameException{
         Hashtable map = brokerObjectName.getKeyPropertyList();
         ObjectName objectName = new ObjectName(brokerObjectName.getDomain() + ":" + "BrokerName=" + map.get("BrokerName") + ","
                             + "Type=SlowConsumerStrategy," + "InstanceName=" + JMXSupport.encodeObjectNamePart(strategy.getName()));
-        return objectName;            
+        return objectName;
     }
 
     public ObjectName getSubscriberObjectName(Subscription key) {
