@@ -28,32 +28,23 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.Map.Entry;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Future;
-import java.util.concurrent.FutureTask;
-import java.util.concurrent.LinkedBlockingQueue;
-import java.util.concurrent.Semaphore;
-import java.util.concurrent.ThreadFactory;
-import java.util.concurrent.ThreadPoolExecutor;
-import java.util.concurrent.TimeUnit;
+import java.util.concurrent.*;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.activemq.broker.ConnectionContext;
+import org.apache.activemq.broker.region.Destination;
+import org.apache.activemq.broker.region.RegionBroker;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQQueue;
 import org.apache.activemq.command.ActiveMQTempQueue;
 import org.apache.activemq.command.ActiveMQTempTopic;
 import org.apache.activemq.command.ActiveMQTopic;
-import org.apache.activemq.command.LocalTransactionId;
 import org.apache.activemq.command.Message;
 import org.apache.activemq.command.MessageAck;
 import org.apache.activemq.command.MessageId;
 import org.apache.activemq.command.ProducerId;
 import org.apache.activemq.command.SubscriptionInfo;
 import org.apache.activemq.command.TransactionId;
-import org.apache.activemq.command.XATransactionId;
-import org.apache.activemq.filter.BooleanExpression;
-import org.apache.activemq.filter.MessageEvaluationContext;
 import org.apache.activemq.openwire.OpenWireFormat;
 import org.apache.activemq.protobuf.Buffer;
 import org.apache.activemq.selector.SelectorParser;
@@ -65,26 +56,25 @@ import org.apache.activemq.store.TopicMessageStore;
 import org.apache.activemq.store.TransactionStore;
 import org.apache.activemq.store.kahadb.data.KahaAddMessageCommand;
 import org.apache.activemq.store.kahadb.data.KahaDestination;
-import org.apache.activemq.store.kahadb.data.KahaLocalTransactionId;
 import org.apache.activemq.store.kahadb.data.KahaLocation;
 import org.apache.activemq.store.kahadb.data.KahaRemoveDestinationCommand;
 import org.apache.activemq.store.kahadb.data.KahaRemoveMessageCommand;
 import org.apache.activemq.store.kahadb.data.KahaSubscriptionCommand;
 import org.apache.activemq.store.kahadb.data.KahaTransactionInfo;
-import org.apache.activemq.store.kahadb.data.KahaXATransactionId;
 import org.apache.activemq.store.kahadb.data.KahaDestination.DestinationType;
 import org.apache.activemq.usage.MemoryUsage;
 import org.apache.activemq.usage.SystemUsage;
 import org.apache.activemq.util.IOExceptionSupport;
 import org.apache.activemq.util.ServiceStopper;
 import org.apache.activemq.wireformat.WireFormat;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.kahadb.util.ByteSequence;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.kahadb.journal.Location;
 import org.apache.kahadb.page.Transaction;
 
 public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
-    static final Log LOG = LogFactory.getLog(KahaDBStore.class);
+    static final Logger LOG = LoggerFactory.getLogger(KahaDBStore.class);
     private static final int MAX_ASYNC_JOBS = 10000;
 
     public static final String PROPERTY_CANCELED_TASK_MOD_METRIC = "org.apache.activemq.store.kahadb.CANCELED_TASK_MOD_METRIC";
@@ -111,9 +101,16 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     private boolean concurrentStoreAndDispatchTransactions = false;
     private int maxAsyncJobs = MAX_ASYNC_JOBS;
     private final KahaDBTransactionStore transactionStore;
+    private TransactionIdTransformer transactionIdTransformer;
 
     public KahaDBStore() {
         this.transactionStore = new KahaDBTransactionStore(this);
+        this.transactionIdTransformer = new TransactionIdTransformer() {
+            @Override
+            public KahaTransactionInfo transform(TransactionId txid) {
+                return TransactionIdConversion.convert(txid);
+            }
+        };
     }
 
     public void setBrokerName(String brokerName) {
@@ -160,7 +157,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     public boolean isConcurrentStoreAndDispatchTransactions() {
         return this.concurrentStoreAndDispatchTransactions;
     }
-    
+
     /**
      * @return the maxAsyncJobs
      */
@@ -182,7 +179,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         this.globalTopicSemaphore = new Semaphore(getMaxAsyncJobs());
         this.asyncQueueJobQueue = new LinkedBlockingQueue<Runnable>(getMaxAsyncJobs());
         this.asyncTopicJobQueue = new LinkedBlockingQueue<Runnable>(getMaxAsyncJobs());
-        this.queueExecutor = new ThreadPoolExecutor(1, asyncExecutorMaxThreads, 0L, TimeUnit.MILLISECONDS,
+        this.queueExecutor = new StoreTaskExecutor(1, asyncExecutorMaxThreads, 0L, TimeUnit.MILLISECONDS,
                 asyncQueueJobQueue, new ThreadFactory() {
                     public Thread newThread(Runnable runnable) {
                         Thread thread = new Thread(runnable, "ConcurrentQueueStoreAndDispatch");
@@ -190,7 +187,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         return thread;
                     }
                 });
-        this.topicExecutor = new ThreadPoolExecutor(1, asyncExecutorMaxThreads, 0L, TimeUnit.MILLISECONDS,
+        this.topicExecutor = new StoreTaskExecutor(1, asyncExecutorMaxThreads, 0L, TimeUnit.MILLISECONDS,
                 asyncTopicJobQueue, new ThreadFactory() {
                     public Thread newThread(Runnable runnable) {
                         Thread thread = new Thread(runnable, "ConcurrentTopicStoreAndDispatch");
@@ -247,6 +244,71 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         super.doStop(stopper);
     }
 
+    void incrementRedeliveryAndReWrite(final String key, final KahaDestination destination) throws IOException {
+        Location location;
+        this.indexLock.writeLock().lock();
+        try {
+              location = findMessageLocation(key, destination);
+        } finally {
+            this.indexLock.writeLock().unlock();
+        }
+
+        if (location != null) {
+            KahaAddMessageCommand addMessage = (KahaAddMessageCommand) load(location);
+            Message message = (Message) wireFormat.unmarshal(new DataInputStream(addMessage.getMessage().newInput()));
+
+            message.incrementRedeliveryCounter();
+            if (LOG.isTraceEnabled()) {
+                LOG.trace("rewriting: " + key + " with deliveryCount: " + message.getRedeliveryCounter());
+            }
+            org.apache.activemq.util.ByteSequence packet = wireFormat.marshal(message);
+            addMessage.setMessage(new Buffer(packet.getData(), packet.getOffset(), packet.getLength()));
+
+            final Location rewriteLocation = journal.write(toByteSequence(addMessage), true);
+
+            this.indexLock.writeLock().lock();
+            try {
+                pageFile.tx().execute(new Transaction.Closure<IOException>() {
+                    public void execute(Transaction tx) throws IOException {
+                        StoredDestination sd = getStoredDestination(destination, tx);
+                        Long sequence = sd.messageIdIndex.get(tx, key);
+                        MessageKeys keys = sd.orderIndex.get(tx, sequence);
+                        sd.orderIndex.put(tx, sd.orderIndex.lastGetPriority(), sequence, new MessageKeys(keys.messageId, rewriteLocation));
+                    }
+                });
+            } finally {
+                this.indexLock.writeLock().unlock();
+            }
+        }
+    }
+
+    @Override
+    void rollbackStatsOnDuplicate(KahaDestination commandDestination) {
+        if (brokerService != null) {
+            RegionBroker regionBroker = (RegionBroker) brokerService.getRegionBroker();
+            if (regionBroker != null) {
+                Set<Destination> destinationSet = regionBroker.getDestinations(convert(commandDestination));
+                for (Destination destination : destinationSet) {
+                    destination.getDestinationStatistics().getMessages().decrement();
+                    destination.getDestinationStatistics().getEnqueues().decrement();
+                }
+            }
+        }
+    }
+
+    private Location findMessageLocation(final String key, final KahaDestination destination) throws IOException {
+        return pageFile.tx().execute(new Transaction.CallableClosure<Location, IOException>() {
+            public Location execute(Transaction tx) throws IOException {
+                StoredDestination sd = getStoredDestination(destination, tx);
+                Long sequence = sd.messageIdIndex.get(tx, key);
+                if (sequence == null) {
+                    return null;
+                }
+                return sd.orderIndex.get(tx, sequence).location;
+            }
+        });
+    }
+
     protected StoreQueueTask removeQueueTask(KahaDBMessageStore store, MessageId id) {
         StoreQueueTask task = null;
         synchronized (store.asyncTaskMap) {
@@ -358,20 +420,20 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
             KahaAddMessageCommand command = new KahaAddMessageCommand();
             command.setDestination(dest);
             command.setMessageId(message.getMessageId().toString());
-            command.setTransactionInfo(createTransactionInfo(message.getTransactionId()));
+            command.setTransactionInfo(transactionIdTransformer.transform(message.getTransactionId()));
             command.setPriority(message.getPriority());
             command.setPrioritySupported(isPrioritizedMessages());
             org.apache.activemq.util.ByteSequence packet = wireFormat.marshal(message);
             command.setMessage(new Buffer(packet.getData(), packet.getOffset(), packet.getLength()));
             store(command, isEnableJournalDiskSyncs() && message.isResponseRequired(), null, null);
-            
+
         }
 
         public void removeMessage(ConnectionContext context, MessageAck ack) throws IOException {
             KahaRemoveMessageCommand command = new KahaRemoveMessageCommand();
             command.setDestination(dest);
             command.setMessageId(ack.getLastMessageId().toString());
-            command.setTransactionInfo(createTransactionInfo(ack.getTransactionId()));
+            command.setTransactionInfo(transactionIdTransformer.transform(ack.getTransactionId()));
 
             org.apache.activemq.util.ByteSequence packet = wireFormat.marshal(ack);
             command.setAck(new Buffer(packet.getData(), packet.getOffset(), packet.getLength()));
@@ -391,20 +453,11 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
             // operations... but for now we must
             // externally synchronize...
             Location location;
-            indexLock.readLock().lock();
+            indexLock.writeLock().lock();
             try {
-                location = pageFile.tx().execute(new Transaction.CallableClosure<Location, IOException>() {
-                    public Location execute(Transaction tx) throws IOException {
-                        StoredDestination sd = getStoredDestination(dest, tx);
-                        Long sequence = sd.messageIdIndex.get(tx, key);
-                        if (sequence == null) {
-                            return null;
-                        }
-                        return sd.orderIndex.get(tx, sequence).location;
-                    }
-                });
+                location = findMessageLocation(key, dest);
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
             if (location == null) {
                 return null;
@@ -416,7 +469,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         public int getMessageCount() throws IOException {
             try {
                 lockAsyncJobQueue();
-                indexLock.readLock().lock();
+                indexLock.writeLock().lock();
                 try {
                     return pageFile.tx().execute(new Transaction.CallableClosure<Integer, IOException>() {
                         public Integer execute(Transaction tx) throws IOException {
@@ -434,7 +487,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         }
                     });
                 }finally {
-                    indexLock.readLock().unlock();
+                    indexLock.writeLock().unlock();
                 }
             } finally {
                 unlockAsyncJobQueue();
@@ -443,7 +496,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
 
         @Override
         public boolean isEmpty() throws IOException {
-            indexLock.readLock().lock();
+            indexLock.writeLock().lock();
             try {
                 return pageFile.tx().execute(new Transaction.CallableClosure<Boolean, IOException>() {
                     public Boolean execute(Transaction tx) throws IOException {
@@ -454,33 +507,37 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                     }
                 });
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
         }
 
         public void recover(final MessageRecoveryListener listener) throws Exception {
-            indexLock.readLock().lock();
+            // recovery may involve expiry which will modify
+            indexLock.writeLock().lock();
             try {
                 pageFile.tx().execute(new Transaction.Closure<Exception>() {
                     public void execute(Transaction tx) throws Exception {
                         StoredDestination sd = getStoredDestination(dest, tx);
                         sd.orderIndex.resetCursorPosition();
-                        for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx); iterator
-                                .hasNext();) {
+                        for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx); listener.hasSpace() && iterator
+                                .hasNext(); ) {
                             Entry<Long, MessageKeys> entry = iterator.next();
+                            if (ackedAndPrepared.contains(entry.getValue().messageId)) {
+                                continue;
+                            }
                             Message msg = loadMessage(entry.getValue().location);
                             listener.recoverMessage(msg);
                         }
                     }
                 });
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
         }
 
-        
+
         public void recoverNextMessages(final int maxReturned, final MessageRecoveryListener listener) throws Exception {
-            indexLock.readLock().lock();
+            indexLock.writeLock().lock();
             try {
                 pageFile.tx().execute(new Transaction.Closure<Exception>() {
                     public void execute(Transaction tx) throws Exception {
@@ -490,6 +547,9 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx);
                              listener.hasSpace() && iterator.hasNext(); ) {
                             entry = iterator.next();
+                            if (ackedAndPrepared.contains(entry.getValue().messageId)) {
+                                continue;
+                            }
                             Message msg = loadMessage(entry.getValue().location);
                             listener.recoverMessage(msg);
                             counter++;
@@ -501,21 +561,26 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                     }
                 });
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
         }
 
         public void resetBatching() {
-            try {
-                pageFile.tx().execute(new Transaction.Closure<Exception>() {
-                    public void execute(Transaction tx) throws Exception {
-                        StoredDestination sd = getExistingStoredDestination(dest, tx);
-                        if (sd != null) {
-                            sd.orderIndex.resetCursorPosition();}
-                        }
-                    });
-            } catch (Exception e) {
-                LOG.error("Failed to reset batching",e);
+            if (pageFile.isLoaded()) {
+                indexLock.writeLock().lock();
+                try {
+                    pageFile.tx().execute(new Transaction.Closure<Exception>() {
+                        public void execute(Transaction tx) throws Exception {
+                            StoredDestination sd = getExistingStoredDestination(dest, tx);
+                            if (sd != null) {
+                                sd.orderIndex.resetCursorPosition();}
+                            }
+                        });
+                } catch (Exception e) {
+                    LOG.error("Failed to reset batching",e);
+                }finally {
+                    indexLock.writeLock().unlock();
+                }
             }
         }
 
@@ -528,10 +593,10 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                 // Hopefully one day the page file supports concurrent read
                 // operations... but for now we must
                 // externally synchronize...
-               
+
                 indexLock.writeLock().lock();
                 try {
-                        pageFile.tx().execute(new Transaction.Closure<IOException>() {
+                    pageFile.tx().execute(new Transaction.Closure<IOException>() {
                         public void execute(Transaction tx) throws IOException {
                             StoredDestination sd = getStoredDestination(dest, tx);
                             Long location = sd.messageIdIndex.get(tx, key);
@@ -540,14 +605,12 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                             }
                         }
                     });
-                }finally {
+                } finally {
                     indexLock.writeLock().unlock();
                 }
-                
             } finally {
                 unlockAsyncJobQueue();
             }
-
         }
 
         @Override
@@ -612,7 +675,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         public void acknowledge(ConnectionContext context, String clientId, String subscriptionName,
                                 MessageId messageId, MessageAck ack)
                 throws IOException {
-            String subscriptionKey = subscriptionKey(clientId, subscriptionName);
+            String subscriptionKey = subscriptionKey(clientId, subscriptionName).toString();
             if (isConcurrentStoreAndDispatchTopics()) {
                 AsyncJobKey key = new AsyncJobKey(messageId, getDestination());
                 StoreTopicTask task = null;
@@ -642,6 +705,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
             command.setDestination(dest);
             command.setSubscriptionKey(subscriptionKey);
             command.setMessageId(messageId.toString());
+            command.setTransactionInfo(transactionIdTransformer.transform(ack.getTransactionId()));
             if (ack != null && ack.isUnmatchedAck()) {
                 command.setAck(UNMATCHED);
             }
@@ -653,7 +717,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                     .getSubscriptionName());
             KahaSubscriptionCommand command = new KahaSubscriptionCommand();
             command.setDestination(dest);
-            command.setSubscriptionKey(subscriptionKey);
+            command.setSubscriptionKey(subscriptionKey.toString());
             command.setRetroactive(retroactive);
             org.apache.activemq.util.ByteSequence packet = wireFormat.marshal(subscriptionInfo);
             command.setSubscriptionInfo(new Buffer(packet.getData(), packet.getOffset(), packet.getLength()));
@@ -664,7 +728,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         public void deleteSubscription(String clientId, String subscriptionName) throws IOException {
             KahaSubscriptionCommand command = new KahaSubscriptionCommand();
             command.setDestination(dest);
-            command.setSubscriptionKey(subscriptionKey(clientId, subscriptionName));
+            command.setSubscriptionKey(subscriptionKey(clientId, subscriptionName).toString());
             store(command, isEnableJournalDiskSyncs() && true, null, null);
             this.subscriptionCount.decrementAndGet();
         }
@@ -672,7 +736,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         public SubscriptionInfo[] getAllSubscriptions() throws IOException {
 
             final ArrayList<SubscriptionInfo> subscriptions = new ArrayList<SubscriptionInfo>();
-            indexLock.readLock().lock();
+            indexLock.writeLock().lock();
             try {
                 pageFile.tx().execute(new Transaction.Closure<IOException>() {
                     public void execute(Transaction tx) throws IOException {
@@ -688,7 +752,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                     }
                 });
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
 
             SubscriptionInfo[] rc = new SubscriptionInfo[subscriptions.size()];
@@ -698,7 +762,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
 
         public SubscriptionInfo lookupSubscription(String clientId, String subscriptionName) throws IOException {
             final String subscriptionKey = subscriptionKey(clientId, subscriptionName);
-            indexLock.readLock().lock();
+            indexLock.writeLock().lock();
             try {
                 return pageFile.tx().execute(new Transaction.CallableClosure<SubscriptionInfo, IOException>() {
                     public SubscriptionInfo execute(Transaction tx) throws IOException {
@@ -712,7 +776,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                     }
                 });
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
         }
 
@@ -723,21 +787,13 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                 return pageFile.tx().execute(new Transaction.CallableClosure<Integer, IOException>() {
                     public Integer execute(Transaction tx) throws IOException {
                         StoredDestination sd = getStoredDestination(dest, tx);
-                        LastAck cursorPos = sd.subscriptionAcks.get(tx, subscriptionKey);
+                        LastAck cursorPos = getLastAck(tx, sd, subscriptionKey);
                         if (cursorPos == null) {
                             // The subscription might not exist.
                             return 0;
                         }
 
-                        int counter = 0;
-                        for (Iterator<Entry<Long, HashSet<String>>> iterator =
-                                sd.ackPositions.iterator(tx, cursorPos.lastAckedSequence); iterator.hasNext();) {
-                            Entry<Long, HashSet<String>> entry = iterator.next();
-                            if (entry.getValue().contains(subscriptionKey)) {
-                                counter++;
-                            }
-                        }
-                        return counter;
+                        return (int) getStoredMessageCount(tx, sd, subscriptionKey);
                     }
                 });
             }finally {
@@ -748,13 +804,14 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         public void recoverSubscription(String clientId, String subscriptionName, final MessageRecoveryListener listener)
                 throws Exception {
             final String subscriptionKey = subscriptionKey(clientId, subscriptionName);
+            @SuppressWarnings("unused")
             final SubscriptionInfo info = lookupSubscription(clientId, subscriptionName);
             indexLock.writeLock().lock();
             try {
                 pageFile.tx().execute(new Transaction.Closure<Exception>() {
                     public void execute(Transaction tx) throws Exception {
                         StoredDestination sd = getStoredDestination(dest, tx);
-                        LastAck cursorPos = sd.subscriptionAcks.get(tx, subscriptionKey);
+                        LastAck cursorPos = getLastAck(tx, sd, subscriptionKey);
                         sd.orderIndex.setBatch(tx, cursorPos);
                         for (Iterator<Entry<Long, MessageKeys>> iterator = sd.orderIndex.iterator(tx); iterator
                                 .hasNext();) {
@@ -772,6 +829,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         public void recoverNextMessages(String clientId, String subscriptionName, final int maxReturned,
                 final MessageRecoveryListener listener) throws Exception {
             final String subscriptionKey = subscriptionKey(clientId, subscriptionName);
+            @SuppressWarnings("unused")
             final SubscriptionInfo info = lookupSubscription(clientId, subscriptionName);
             indexLock.writeLock().lock();
             try {
@@ -781,7 +839,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                         sd.orderIndex.resetCursorPosition();
                         MessageOrderCursor moc = sd.subscriptionCursors.get(subscriptionKey);
                         if (moc == null) {
-                            LastAck pos = sd.subscriptionAcks.get(tx, subscriptionKey);
+                            LastAck pos = getLastAck(tx, sd, subscriptionKey);
                             if (pos == null) {
                                 // sub deleted
                                 return;
@@ -851,7 +909,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     /**
      * Cleanup method to remove any state associated with the given destination.
      * This method does not stop the message store (it might not be cached).
-     * 
+     *
      * @param destination
      *            Destination to forget
      */
@@ -861,7 +919,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     /**
      * Cleanup method to remove any state associated with the given destination
      * This method does not stop the message store (it might not be cached).
-     * 
+     *
      * @param destination
      *            Destination to forget
      */
@@ -875,7 +933,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     public Set<ActiveMQDestination> getDestinations() {
         try {
             final HashSet<ActiveMQDestination> rc = new HashSet<ActiveMQDestination>();
-            indexLock.readLock().lock();
+            indexLock.writeLock().lock();
             try {
                 pageFile.tx().execute(new Transaction.Closure<IOException>() {
                     public void execute(Transaction tx) throws IOException {
@@ -902,7 +960,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                     }
                 });
             }finally {
-                indexLock.readLock().unlock();
+                indexLock.writeLock().unlock();
             }
             return rc;
         } catch (IOException e) {
@@ -913,7 +971,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
     public long getLastMessageBrokerSequenceId() throws IOException {
         return 0;
     }
-    
+
     public long getLastProducerSequenceId(ProducerId id) {
         indexLock.readLock().lock();
         try {
@@ -995,7 +1053,14 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         }
         int type = Integer.parseInt(dest.substring(0, p));
         String name = dest.substring(p + 1);
+        return convert(type, name);
+    }
 
+    private ActiveMQDestination convert(KahaDestination commandDestination) {
+        return convert(commandDestination.getType().getNumber(), commandDestination.getName());
+    }
+
+    private ActiveMQDestination convert(int type, String name) {
         switch (KahaDestination.DestinationType.valueOf(type)) {
         case QUEUE:
             return new ActiveMQQueue(name);
@@ -1010,6 +1075,14 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         }
     }
 
+    public TransactionIdTransformer getTransactionIdTransformer() {
+        return transactionIdTransformer;
+    }
+
+    public void setTransactionIdTransformer(TransactionIdTransformer transactionIdTransformer) {
+        this.transactionIdTransformer = transactionIdTransformer;
+    }
+
     static class AsyncJobKey {
         MessageId id;
         ActiveMQDestination destination;
@@ -1039,8 +1112,12 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         }
     }
 
-    interface StoreTask {
+    public interface StoreTask {
         public boolean cancel();
+
+        public void aquireLocks();
+
+        public void releaseLocks();
     }
 
     class StoreQueueTask implements Runnable, StoreTask {
@@ -1063,14 +1140,13 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         }
 
         public boolean cancel() {
-            releaseLocks();
             if (this.done.compareAndSet(false, true)) {
                 return this.future.cancel(false);
             }
             return false;
         }
 
-        void aquireLocks() {
+        public void aquireLocks() {
             if (this.locked.compareAndSet(false, true)) {
                 try {
                     globalQueueSemaphore.acquire();
@@ -1083,7 +1159,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
 
         }
 
-        void releaseLocks() {
+        public void releaseLocks() {
             if (this.locked.compareAndSet(true, false)) {
                 store.releaseLocalAsyncLock();
                 globalQueueSemaphore.release();
@@ -1105,8 +1181,6 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                 }
             } catch (Exception e) {
                 this.future.setException(e);
-            } finally {
-                releaseLocks();
             }
         }
 
@@ -1144,7 +1218,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         }
 
         @Override
-        void aquireLocks() {
+        public void aquireLocks() {
             if (this.locked.compareAndSet(false, true)) {
                 try {
                     globalTopicSemaphore.acquire();
@@ -1158,7 +1232,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
         }
 
         @Override
-        void releaseLocks() {
+        public void releaseLocks() {
             if (this.locked.compareAndSet(true, false)) {
                 message.decrementReferenceCount();
                 store.releaseLocalAsyncLock();
@@ -1168,7 +1242,7 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
 
         /**
          * add a key
-         * 
+         *
          * @param key
          * @return true if all acknowledgements received
          */
@@ -1201,9 +1275,23 @@ public class KahaDBStore extends MessageDatabase implements PersistenceAdapter {
                 }
             } catch (Exception e) {
                 this.future.setException(e);
-            } finally {
-                releaseLocks();
             }
         }
     }
+
+    public class StoreTaskExecutor extends ThreadPoolExecutor {
+
+        public StoreTaskExecutor(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit timeUnit, BlockingQueue<Runnable> queue, ThreadFactory threadFactory) {
+            super(corePoolSize, maximumPoolSize, keepAliveTime, timeUnit, queue, threadFactory);
+        }
+
+        protected void afterExecute(Runnable runnable, Throwable throwable) {
+            super.afterExecute(runnable, throwable);
+
+            if (runnable instanceof StoreTask) {
+               ((StoreTask)runnable).releaseLocks();
+            }
+
+        }
+    }
 }
