@@ -25,6 +25,8 @@ import java.util.Map;
 
 import javax.jms.BytesMessage;
 import javax.jms.Connection;
+import javax.jms.ConnectionFactory;
+import javax.jms.Destination;
 import javax.jms.Message;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
@@ -47,21 +49,22 @@ import org.apache.activemq.broker.region.BaseDestination;
 import org.apache.activemq.broker.region.policy.PolicyEntry;
 import org.apache.activemq.broker.region.policy.PolicyMap;
 import org.apache.activemq.broker.region.policy.SharedDeadLetterStrategy;
-import org.apache.activemq.command.ActiveMQBlobMessage;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQQueue;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.activemq.command.ActiveMQTempQueue;
+import org.apache.activemq.util.JMXSupport;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * A test case of the various MBeans in ActiveMQ. If you want to look at the
  * various MBeans after the test has been run then run this test case as a
  * command line application.
- * 
- * @version $Revision$
+ *
+ *
  */
 public class MBeanTest extends EmbeddedBrokerTestSupport {
-    private static final Log LOG = LogFactory.getLog(MBeanTest.class);
+    private static final Logger LOG = LoggerFactory.getLogger(MBeanTest.class);
 
     private static boolean waitForKeyPress;
 
@@ -83,12 +86,13 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         waitForKeyPress = true;
         TestRunner.run(MBeanTest.class);
     }
-    
+
     public void testConnectors() throws Exception{
         ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
         BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
-        assertEquals("openwire URL port doesn't equal bind Address", new URI(broker.getOpenWireURL()).getPort(), new URI(this.bindAddress).getPort());
-
+        assertEquals("openwire URL port doesn't equal bind Address",
+                     new URI(broker.getOpenWireURL()).getPort(),
+                     new URI(this.broker.getTransportConnectors().get(0).getPublishableConnectString()).getPort());
     }
 
     public void testMBeans() throws Exception {
@@ -101,6 +105,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertQueueBrowseWorks();
         assertCreateAndDestroyDurableSubscriptions();
         assertConsumerCounts();
+        assertProducerCounts();
     }
 
     public void testMoveMessages() throws Exception {
@@ -152,7 +157,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         long newQueuesize = queueNew.getQueueSize();
         echo("Second queue size: " + newQueuesize);
         assertEquals("Unexpected number of messages ",messageCount, newQueuesize);
-        
+
         // check memory usage migration
         assertTrue("new dest has some memory usage", queueNew.getMemoryPercentUsage() > 0);
         assertEquals("old dest has no memory usage", 0, queue.getMemoryPercentUsage());
@@ -254,7 +259,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         int dlqMemUsage = dlq.getMemoryPercentUsage();
         assertTrue("dlq has some memory usage", dlqMemUsage > 0);
         assertEquals("dest has no memory usage", 0, queue.getMemoryPercentUsage());
-        
+
 
         echo("About to retry " + messageCount + " messages");
 
@@ -275,7 +280,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("DLQ size", initialDlqSize - messageCount, dlqSize);
         assertEquals("queue size", initialQueueSize, queueSize);
         assertEquals("browse queue size", initialQueueSize, actualCount);
-        
+
         assertEquals("dest has some memory usage", dlqMemUsage, queue.getMemoryPercentUsage());
     }
 
@@ -313,10 +318,9 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
 
         String newDestination = getSecondDestinationString();
         long queueSize = queue.getQueueSize();
+        assertTrue(queueSize > 0);
         queue.copyMatchingMessagesTo("counter > 2", newDestination);
 
-        
-
         queueViewMBeanName = assertRegisteredObjectName(domain + ":Type=Queue,Destination=" + newDestination + ",BrokerName=localhost");
 
         queue = (QueueViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, queueViewMBeanName, QueueViewMBean.class, true);
@@ -330,7 +334,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("dest has no memory usage", 0, queue.getMemoryPercentUsage());
     }
 
-
+    @SuppressWarnings("rawtypes")
     protected void assertSendViaMBean() throws Exception {
         String queueName = getDestinationString() + ".SendMBBean";
 
@@ -345,12 +349,12 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         QueueViewMBean proxy = (QueueViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, queueViewMBeanName, QueueViewMBean.class, true);
 
         proxy.purge();
-        
+
         int count = 5;
         for (int i = 0; i < count; i++) {
             String body = "message:" + i;
 
-            Map headers = new HashMap();
+            Map<String, Object> headers = new HashMap<String, Object>();
             headers.put("JMSCorrelationID", "MyCorrId");
             headers.put("JMSDeliveryMode", Boolean.FALSE);
             headers.put("JMSXGroupID", "MyGroupID");
@@ -362,12 +366,11 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
 
             proxy.sendTextMessage(headers, body);
         }
-        
+
         CompositeData[] compdatalist = proxy.browse();
         if (compdatalist.length == 0) {
             fail("There is no message in the queue:");
         }
-        String[] messageIDs = new String[compdatalist.length];
 
         for (int i = 0; i < compdatalist.length; i++) {
             CompositeData cdata = compdatalist[i];
@@ -404,7 +407,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
             assertComplexData(i, cdata, "JMSXGroupSeq", 1234);
             assertComplexData(i, cdata, "JMSXGroupID", "MyGroupID");
             assertComplexData(i, cdata, "Text", "message:" + i);
-
         }
     }
 
@@ -544,6 +546,80 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("topic2 Durable subscriber count", 0, topic2.getConsumerCount());
     }
 
+    protected void assertProducerCounts() throws Exception {
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        assertTrue("broker is not a slave", !broker.isSlave());
+        // create 2 topics
+        broker.addTopic(getDestinationString() + "1");
+        broker.addTopic(getDestinationString() + "2");
+
+        ObjectName topicObjName1 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "1");
+        ObjectName topicObjName2 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "2");
+        TopicViewMBean topic1 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName1, TopicViewMBean.class, true);
+        TopicViewMBean topic2 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName2, TopicViewMBean.class, true);
+
+        assertEquals("topic1 Producer count", 0, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 0, topic2.getProducerCount());
+        assertEquals("broker Topic Producer count", 0, broker.getTopicProducers().length);
+
+        // create 1 producer for each topic
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        Destination dest1 = session.createTopic(getDestinationString() + "1");
+        Destination dest2 = session.createTopic(getDestinationString() + "2");
+        MessageProducer producer1 = session.createProducer(dest1);
+        MessageProducer producer2 = session.createProducer(dest2);
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 1, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 1, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 2, broker.getTopicProducers().length);
+
+        // create 1 more producer for topic1
+        MessageProducer producer3 = session.createProducer(dest1);
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 2, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 1, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 3, broker.getTopicProducers().length);
+
+        // destroy topic1 producer
+        producer1.close();
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 1, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 1, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 2, broker.getTopicProducers().length);
+
+        // destroy topic2 producer
+        producer2.close();
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 1, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 0, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 1, broker.getTopicProducers().length);
+
+        // destroy remaining topic1 producer
+        producer3.close();
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 0, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 0, topic2.getProducerCount());
+
+        MessageProducer producer4 = session.createProducer(null);
+        Thread.sleep(500);
+        assertEquals(1, broker.getDynamicDestinationProducers().length);
+        producer4.close();
+        Thread.sleep(500);
+
+        assertEquals("broker Topic Producer count", 0, broker.getTopicProducers().length);
+    }
+
     protected ObjectName assertRegisteredObjectName(String name) throws MalformedObjectNameException, NullPointerException {
         ObjectName objectName = new ObjectName(name);
         if (mbeanServer.isRegistered(objectName)) {
@@ -555,7 +631,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
     }
 
     protected void setUp() throws Exception {
-        bindAddress = "tcp://localhost:61616";
+        bindAddress = "tcp://localhost:0";
         useTopic = false;
         super.setUp();
         mbeanServer = broker.getManagementContext().getMBeanServer();
@@ -579,19 +655,24 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         super.tearDown();
     }
 
+    @Override
+    protected ConnectionFactory createConnectionFactory() throws Exception {
+        return new ActiveMQConnectionFactory(broker.getTransportConnectors().get(0).getPublishableConnectString());
+    }
+
     protected BrokerService createBroker() throws Exception {
         BrokerService answer = new BrokerService();
         answer.setPersistent(false);
         answer.setDeleteAllMessagesOnStartup(true);
         answer.setUseJmx(true);
-       
+
         // apply memory limit so that %usage is visible
         PolicyMap policyMap = new PolicyMap();
         PolicyEntry defaultEntry = new PolicyEntry();
         defaultEntry.setMemoryLimit(1024*1024*4);
         policyMap.setDefaultEntry(defaultEntry);
         answer.setDestinationPolicy(policyMap);
-        
+
         answer.addConnector(bindAddress);
         return answer;
     }
@@ -614,7 +695,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         Thread.sleep(1000);
     }
 
-    
     protected void useConnectionWithBlobMessage(Connection connection) throws Exception {
         connection.setClientID(clientID);
         connection.start();
@@ -656,11 +736,89 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         LOG.info(text);
     }
 
-
     protected String getSecondDestinationString() {
         return "test.new.destination." + getClass() + "." + getName();
     }
 
+    public void testDynamicProducerView() throws Exception {
+        connection = connectionFactory.createConnection();
+
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        assertTrue("broker is not a slave", !broker.isSlave());
+        assertEquals(0, broker.getDynamicDestinationProducers().length);
+
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        MessageProducer producer = session.createProducer(null);
+
+        Destination dest1 = session.createTopic("DynamicDest-1");
+        Destination dest2 = session.createTopic("DynamicDest-2");
+        Destination dest3 = session.createQueue("DynamicDest-3");
+
+        // Wait a bit to let the producer get registered.
+        Thread.sleep(100);
+
+        assertEquals(1, broker.getDynamicDestinationProducers().length);
+
+        ObjectName viewName = broker.getDynamicDestinationProducers()[0];
+        assertNotNull(viewName);
+        ProducerViewMBean view = (ProducerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, viewName, ProducerViewMBean.class, true);
+        assertNotNull(view);
+
+        assertEquals("NOTSET", view.getDestinationName());
+
+        producer.send(dest1, session.createTextMessage("Test Message 1"));
+        Thread.sleep(200);
+        assertEquals(((ActiveMQDestination)dest1).getPhysicalName(), view.getDestinationName());
+        assertTrue(view.isDestinationTopic());
+        assertFalse(view.isDestinationQueue());
+        assertFalse(view.isDestinationTemporary());
+
+        producer.send(dest2, session.createTextMessage("Test Message 2"));
+        Thread.sleep(200);
+        assertEquals(((ActiveMQDestination)dest2).getPhysicalName(), view.getDestinationName());
+        assertTrue(view.isDestinationTopic());
+        assertFalse(view.isDestinationQueue());
+        assertFalse(view.isDestinationTemporary());
+
+        producer.send(dest3, session.createTextMessage("Test Message 3"));
+        Thread.sleep(200);
+        assertEquals(((ActiveMQDestination)dest3).getPhysicalName(), view.getDestinationName());
+        assertTrue(view.isDestinationQueue());
+        assertFalse(view.isDestinationTopic());
+        assertFalse(view.isDestinationTemporary());
+
+        producer.close();
+        Thread.sleep(200);
+        assertEquals(0, broker.getDynamicDestinationProducers().length);
+    }
+
+    public void testTempQueueJMXDelete() throws Exception {
+        connection = connectionFactory.createConnection();
+
+        connection.setClientID(clientID);
+        connection.start();
+        Session session = connection.createSession(transacted, authMode);
+        ActiveMQTempQueue tQueue = (ActiveMQTempQueue) session.createTemporaryQueue();
+        Thread.sleep(1000);
+        ObjectName queueViewMBeanName = assertRegisteredObjectName(domain + ":Type="+  JMXSupport.encodeObjectNamePart(tQueue.getDestinationTypeAsString())+",Destination=" + JMXSupport.encodeObjectNamePart(tQueue.getPhysicalName()) + ",BrokerName=localhost");
+
+        // should not throw an exception
+        mbeanServer.getObjectInstance(queueViewMBeanName);
+
+        tQueue.delete();
+        Thread.sleep(1000);
+        try {
+            // should throw an exception
+            mbeanServer.getObjectInstance(queueViewMBeanName);
+
+            fail("should be deleted already!");
+        } catch (Exception e) {
+            // expected!
+        }
+    }
+
     // Test for AMQ-3029
     public void testBrowseBlobMessages() throws Exception {
         connection = connectionFactory.createConnection();
@@ -684,7 +842,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
             CompositeData cdata = compdatalist[i];
             String messageID = (String) cdata.get("JMSMessageID");
             assertNotNull("Should have a message ID for message " + i, messageID);
-            
+
             messageIDs[i] = messageID;
         }
 
