@@ -38,8 +38,8 @@ import org.apache.activemq.broker.BrokerService;
 import org.apache.activemq.command.ActiveMQQueue;
 import org.apache.activemq.network.NetworkConnector;
 import org.apache.activemq.store.memory.MemoryPersistenceAdapter;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.junit.Test;
 import org.springframework.jms.connection.CachingConnectionFactory;
 import org.springframework.jms.connection.SingleConnectionFactory;
@@ -48,7 +48,7 @@ import org.springframework.jms.core.MessageCreator;
 import org.springframework.jms.listener.DefaultMessageListenerContainer;
 
 public class LoadBalanceTest {
-    private static final Log LOG = LogFactory.getLog(LoadBalanceTest.class);
+    private static final Logger LOG = LoggerFactory.getLogger(LoadBalanceTest.class);
     private static final String TESTING_QUEUE = "testingqueue";
     private static int networkBridgePrefetch = 1000;
 
@@ -76,6 +76,7 @@ public class LoadBalanceTest {
                 network1.setPrefetchSize(networkBridgePrefetch);
                 network1.setConduitSubscriptions(false);
                 network1.setDecreaseNetworkConsumerPriority(false);
+                network1.setDispatchAsync(false);
                 brokerService1.start();
             }
             {
@@ -93,6 +94,7 @@ public class LoadBalanceTest {
                 network2.setPrefetchSize(networkBridgePrefetch);
                 network2.setConduitSubscriptions(false);
                 network2.setDecreaseNetworkConsumerPriority(false);
+                network2.setDispatchAsync(false);
                 brokerService2.start();
             }
             final ExecutorService pool = Executors.newSingleThreadExecutor();
@@ -174,11 +176,11 @@ public class LoadBalanceTest {
                     return null;
                 }
             });
-            pool.shutdown();
 
-            waitForBridgeFormation(10000);
+            waitForBridgeFormation();
             startProducer.countDown();
 
+            pool.shutdown();
             pool.awaitTermination(10, TimeUnit.SECONDS);
             LOG.info("broker1Count " + broker1Count.get() + ", broker2Count " + broker2Count.get());
 
@@ -289,7 +291,7 @@ public class LoadBalanceTest {
         });
         
         // give network a chance to build, needs advisories
-        waitForBridgeFormation(10000);
+        waitForBridgeFormation();
         startProducer.countDown();
         
         pool.shutdown();
@@ -313,16 +315,24 @@ public class LoadBalanceTest {
 
     // need to ensure broker bridge is alive before starting the consumer
     // peeking at the internals will give us this info
-    private void waitForBridgeFormation(long delay) throws Exception {
-        long done = System.currentTimeMillis() + delay;
+    private void waitForBridgeFormation() throws Exception {
+        long done = System.currentTimeMillis() + 30000;
         while (done > System.currentTimeMillis()) {
-            BrokerService broker = BrokerRegistry.getInstance().lookup("two");
-            if (broker != null && !broker.getNetworkConnectors().isEmpty()) {
-                 if (!broker.getNetworkConnectors().get(0).activeBridges().isEmpty()) {
-                     return;
-                 }
+            if (hasBridge("one") && hasBridge("two")) {
+                return;
             }
             Thread.sleep(1000);
         }
     }
+
+    private boolean hasBridge(String name) {
+        boolean result = false;
+        BrokerService broker = BrokerRegistry.getInstance().lookup(name);
+            if (broker != null && !broker.getNetworkConnectors().isEmpty()) {
+                 if (!broker.getNetworkConnectors().get(0).activeBridges().isEmpty()) {
+                    result = true;
+                 }
+            }
+        return result;
+    }
 }
